@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -135,10 +135,10 @@ SBMLTransforms::replaceBvars(ASTNode * node, const FunctionDefinition *fd)
   ASTNode fdMath(AST_UNKNOWN);
   unsigned int noBvars;
 
-  if (fd->isSetMath() && fd->getMath()->getRightChild() != NULL)
+  if (fd->isSetMath() && fd->getBody() != NULL)
   {
     noBvars = fd->getMath()->getNumBvars();
-    fdMath = *fd->getMath()->getRightChild();
+    fdMath = *fd->getBody();
 
     for (unsigned int i = 0, nodeCount = 0; i < noBvars; i++, nodeCount++)
     {
@@ -158,10 +158,12 @@ bool
 SBMLTransforms::checkFunctionNodeForIds(ASTNode * node, IdList& ids)
 {
   bool present = false;
+  if (node == NULL) return present;
+
   unsigned int i = 0;
   unsigned int numChildren = node->getNumChildren();
 
-  if (node != NULL && node->getType() == AST_FUNCTION)
+  if (node->getType() == AST_FUNCTION)
   {
     if (ids.contains(node->getName()))
     {
@@ -182,6 +184,9 @@ SBMLTransforms::checkFunctionNodeForIds(ASTNode * node, IdList& ids)
 bool
 SBMLTransforms::nodeContainsId(const ASTNode * node, IdList& ids)
 {
+  if (node == NULL) {
+    return false;
+  }
   bool present = false;
   unsigned int i = 0;
   unsigned int numChildren = node->getNumChildren();
@@ -206,6 +211,9 @@ SBMLTransforms::nodeContainsId(const ASTNode * node, IdList& ids)
 bool
 SBMLTransforms::nodeContainsNameNotInList(const ASTNode * node, IdList& ids)
 {
+  if (node == NULL) {
+    return false;
+  }
   bool notInList = false;
   unsigned int i = 0;
   unsigned int numChildren = node->getNumChildren();
@@ -457,6 +465,11 @@ SBMLTransforms::getComponentValuesForModel(const Model * m, IdValueMap& values)
         values.insert(pair<const std::string, ValueSet>(sr->getId(), v));
       }
     }
+
+    // the id of a reaction might be used
+    ValueSet v = make_pair(numeric_limits<double>::quiet_NaN(), true);
+    values.insert(pair<const std::string, ValueSet>(rn->getId(), v));
+
   }
 
   /* returns a list of ids for which the model has no declared value
@@ -497,8 +510,13 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const std::map<std::string
 double
 SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values, const Model * m)
 {
-  double result;
+  if (node == NULL) {
+    return numeric_limits<double>::quiet_NaN();
+  }
+  double result = 0;
   int i;
+  const ListOfFunctionDefinitions *lfd = NULL;
+  ASTNode *tempNode = NULL;
 
   switch (node->getType())
   {
@@ -533,6 +551,7 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
             const Rule *r = m->getRule(node->getName());
             const InitialAssignment *ia = 
                                 m->getInitialAssignment(node->getName());
+            const Reaction *rn = m->getReaction(node->getName()); 
             if (r != NULL)
             {
               result = evaluateASTNode(r->getMath(), values, m);
@@ -541,6 +560,10 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
             {
               result = evaluateASTNode(ia->getMath(), values, m);
             }
+            else if (rn != NULL && rn->isSetKineticLaw())
+            {
+              result = evaluateASTNode(rn->getKineticLaw()->getMath(), values, m);
+            }
           }
         }
       }
@@ -580,11 +603,19 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     break;
 
   case AST_LAMBDA:
-  case AST_FUNCTION:
     /* shouldnt get here */
     result = numeric_limits<double>::quiet_NaN();
     break;
 
+  case AST_FUNCTION:
+    /* shouldnt get here */
+    // but we do if math we are expanding uses a functionDefinition
+    lfd = m->getListOfFunctionDefinitions();
+    tempNode = node->deepCopy();
+    replaceFD(tempNode, lfd);
+    result = evaluateASTNode(tempNode, values, m);
+    delete tempNode;
+    break;
   case AST_PLUS:
     if (node->getNumChildren() == 0)
     {
@@ -597,9 +628,9 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     else
     {
       result = evaluateASTNode(node->getChild(0), values, m);
-      for (unsigned int i = 1; i < node->getNumChildren(); i++)
+      for (unsigned int j = 1; j < node->getNumChildren(); ++j)
       {
-        result = result + evaluateASTNode(node->getChild(i), values, m) ;
+        result = result + evaluateASTNode(node->getChild(j), values, m) ;
       }
     }
     break;
@@ -624,9 +655,9 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     else
     {
       result = evaluateASTNode(node->getChild(0), values, m);
-      for (unsigned int i = 1; i < node->getNumChildren(); i++)
+      for (unsigned int j = 1; j < node->getNumChildren(); ++j)
       {
-        result = result * evaluateASTNode(node->getChild(i), values, m) ;
+        result = result * evaluateASTNode(node->getChild(j), values, m) ;
       }
     }
     break;
@@ -779,11 +810,11 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
       if ( numChildren % 2 == 0)
       {
         bool assigned = false;
-        for (unsigned int i = 0; i < numChildren; i+=2)
+        for (unsigned int j = 0; j < numChildren; j+=2)
         {
           // compute piece
-          double value = evaluateASTNode(node->getChild(i), values, m);
-          double boolean = evaluateASTNode(node->getChild(i + 1), values, m);
+          double value = evaluateASTNode(node->getChild(j), values, m);
+          double boolean = evaluateASTNode(node->getChild(j + 1), values, m);
           if (boolean == 1.0)
           {
             // we might have two true piece statements
@@ -815,11 +846,11 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
       else
       {
         bool assigned = false;
-        for (unsigned int i = 0; i < numChildren-1; i+=2)
+        for (unsigned int j = 0; j < numChildren-1; j+=2)
         {
           // compute piece
-          double value = evaluateASTNode(node->getChild(i), values, m);
-          double boolean = evaluateASTNode(node->getChild(i + 1), values, m);
+          double value = evaluateASTNode(node->getChild(j), values, m);
+          double boolean = evaluateASTNode(node->getChild(j + 1), values, m);
           if (boolean == 1.0)
           {
             // we might have two true piece statements
@@ -930,9 +961,9 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     else 
     {      
       result = 1.0;
-      for (unsigned int i = 1; i < node->getNumChildren(); ++i)
-        result *= (double)((evaluateASTNode(node->getChild(i-1), values, m))
-        == (evaluateASTNode(node->getChild(i), values, m)));
+      for (unsigned int j = 1; j < node->getNumChildren(); ++j)
+        result *= (double)((evaluateASTNode(node->getChild(j-1), values, m))
+        == (evaluateASTNode(node->getChild(j), values, m)));
     }
     break;
 
@@ -941,9 +972,9 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     else 
     {      
       result = 1.0;
-      for (unsigned int i = 1; i < node->getNumChildren(); ++i)
-        result *= (double)((evaluateASTNode(node->getChild(i-1), values, m))
-        >= (evaluateASTNode(node->getChild(i), values, m)));
+      for (unsigned int j = 1; j < node->getNumChildren(); ++j)
+        result *= (double)((evaluateASTNode(node->getChild(j-1), values, m))
+        >= (evaluateASTNode(node->getChild(j), values, m)));
     }
     break;
 
@@ -952,9 +983,9 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     else 
     {      
       result = 1.0;
-      for (unsigned int i = 1; i < node->getNumChildren(); ++i)
-        result *= (double)((evaluateASTNode(node->getChild(i-1), values, m))
-        > (evaluateASTNode(node->getChild(i), values, m)));
+      for (unsigned int j = 1; j < node->getNumChildren(); ++j)
+        result *= (double)((evaluateASTNode(node->getChild(j-1), values, m))
+        > (evaluateASTNode(node->getChild(j), values, m)));
     }
     break;
 
@@ -963,9 +994,9 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     else 
     {      
       result = 1.0;
-      for (unsigned int i = 1; i < node->getNumChildren(); ++i)
-         result *= (double)((evaluateASTNode(node->getChild(i-1), values, m))
-         <= (evaluateASTNode(node->getChild(i), values, m)));
+      for (unsigned int j = 1; j < node->getNumChildren(); ++j)
+         result *= (double)((evaluateASTNode(node->getChild(j-1), values, m))
+         <= (evaluateASTNode(node->getChild(j), values, m)));
     }
     break;
 
@@ -974,9 +1005,9 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     else 
     {      
       result = 1.0;
-      for (unsigned int i = 1; i < node->getNumChildren(); ++i)
-        result *= (double)((evaluateASTNode(node->getChild(i-1), values, m))
-        < (evaluateASTNode(node->getChild(i), values, m)));
+      for (unsigned int j = 1; j < node->getNumChildren(); ++j)
+        result *= (double)((evaluateASTNode(node->getChild(j-1), values, m))
+        < (evaluateASTNode(node->getChild(j), values, m)));
     }
     break;
 
@@ -985,12 +1016,64 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
     else 
     {      
       result = 1.0;
-      for (unsigned int i = 1; i < node->getNumChildren(); ++i)
-        result *= (double)((evaluateASTNode(node->getChild(i-1), values, m))
-        != (evaluateASTNode(node->getChild(i), values, m)));
+      for (unsigned int j = 1; j < node->getNumChildren(); ++j)
+        result *= (double)((evaluateASTNode(node->getChild(j-1), values, m))
+        != (evaluateASTNode(node->getChild(j), values, m)));
     }
     break;
 
+  case AST_FUNCTION_QUOTIENT:
+    if (node->getNumChildren() < 2) result = 0.0;
+    else 
+    {      
+      result = floor(evaluateASTNode(node->getChild(0), values, m) /
+        evaluateASTNode(node->getChild(1), values, m));
+    }
+    break;
+
+  case AST_FUNCTION_REM:
+    if (node->getNumChildren() < 2) result = 0.0;
+    else 
+    {   
+      double dividend = evaluateASTNode(node->getChild(0), values);
+      double divisor = evaluateASTNode(node->getChild(1), values);
+      double quotient = floor(dividend/divisor);
+
+      result = dividend - (quotient * divisor);
+    }
+    break;
+
+  case AST_FUNCTION_MIN:
+    result = evaluateASTNode(node->getChild(0), values);
+    for (unsigned int i = 1; i < node->getNumChildren(); i++)
+    {
+      double nextValue = evaluateASTNode(node->getChild(i), values);
+      if (nextValue < result) result = nextValue;
+    }
+    break;
+
+  case AST_FUNCTION_MAX:
+    result = evaluateASTNode(node->getChild(0), values);
+    for (unsigned int i = 1; i < node->getNumChildren(); i++)
+    {
+      double nextValue = evaluateASTNode(node->getChild(i), values);
+      if (nextValue > result) result = nextValue;
+    }
+    break;
+
+  case AST_LOGICAL_IMPLIES:
+    {
+      if (node->getNumChildren() == 0)
+        result = 0.0;
+      else if (node->getNumChildren() == 1)
+        result = evaluateASTNode(node->getChild(0), values, m);
+      else
+        result = (double)((!(evaluateASTNode(node->getChild(0), values, m)))
+        || (evaluateASTNode(node->getChild(1), values, m)));
+    }
+    break;
+
+  case AST_FUNCTION_RATE_OF:
   default:
     result = numeric_limits<double>::quiet_NaN();
     break;
@@ -1000,6 +1083,73 @@ SBMLTransforms::evaluateASTNode(const ASTNode * node, const IdValueMap& values,
   return result;
 }
 
+bool
+SBMLTransforms::expandIA(Model* m, const InitialAssignment* ia)
+{
+  bool removed = false;
+  std::string id = ia->getSymbol();
+  if (m->getCompartment(id) != NULL) 
+  {
+    if (expandInitialAssignment(m->getCompartment(id), 
+                                ia))
+    {
+      delete m->removeInitialAssignment(id);
+      removed = true;
+    }
+  }
+  else if (m->getParameter(id) != NULL)
+  {
+    if (expandInitialAssignment(m->getParameter(id), 
+                                ia))
+    {
+      delete m->removeInitialAssignment(id);
+      removed = true;
+    }
+  }
+  else if (m->getSpecies(id) != NULL)
+  {
+    if (expandInitialAssignment(m->getSpecies(id), 
+                                ia))
+    {
+      delete m->removeInitialAssignment(id);
+      removed = true;
+    }
+  }
+  else 
+  {
+    for (unsigned int j = 0; j < m->getNumReactions(); j++)
+    {
+      Reaction * r = m->getReaction(j);
+      unsigned int k;
+      for (k = 0; k < r->getNumProducts(); k++)
+      {
+        if (r->getProduct(k)->getId() == id)
+        {
+          if (expandInitialAssignment(r->getProduct(k), 
+                                      ia))
+          {
+            delete m->removeInitialAssignment(id);
+            removed = true;
+          }
+        }
+      }
+      for (k = 0; k < r->getNumReactants(); k++)
+      {
+        if (r->getReactant(k)->getId() == id)
+        {
+          if (expandInitialAssignment(r->getReactant(k), 
+                                      ia))
+          {
+            delete m->removeInitialAssignment(id);
+            removed = true;
+          }
+        }
+      }
+    }
+  }
+
+  return removed;
+}
 
 bool 
 SBMLTransforms::expandInitialAssignments(Model * m)
@@ -1009,12 +1159,13 @@ SBMLTransforms::expandInitialAssignments(Model * m)
 
   IdValueIter iter;
   bool needToBail = false;
+  unsigned int count;
 
   do
   {
     /* need a fail safe in case a value is just missing */
     unsigned int num = m->getNumInitialAssignments();
-    unsigned int count = num;
+    count = num;
     
     /* list ids that have a calculated/assigned value */
     idsWithValues.clear();
@@ -1026,78 +1177,27 @@ SBMLTransforms::expandInitialAssignments(Model * m)
       }
     }
 
-    for (unsigned int i = 0; i < m->getNumInitialAssignments(); i++)
+    for (int i = num-1; i >= 0; i--)
     {
-      if (!nodeContainsId(m->getInitialAssignment(i)->getMath(), idsNoValues))
+      if (m->getInitialAssignment(i)->isSetMath())
       {
-        if (!nodeContainsNameNotInList(m->getInitialAssignment(i)->getMath(), 
-                                                                 idsWithValues))
+        if (!nodeContainsId(m->getInitialAssignment(i)->getMath(), idsNoValues))
         {
-          std::string id = m->getInitialAssignment(i)->getSymbol();
-          if (m->getCompartment(id) != NULL) 
+          if (!nodeContainsNameNotInList(m->getInitialAssignment(i)->getMath(), 
+                                                                   idsWithValues))
           {
-            if (expandInitialAssignment(m->getCompartment(id), 
-                                        m->getInitialAssignment(i)))
-            {
-              delete m->removeInitialAssignment(id);
-              count--;
-            }
-          }
-          else if (m->getParameter(id) != NULL)
-          {
-            if (expandInitialAssignment(m->getParameter(id), 
-                                        m->getInitialAssignment(i)))
-            {
-              delete m->removeInitialAssignment(id);
-              count--;
-            }
-          }
-          else if (m->getSpecies(id) != NULL)
-          {
-            if (expandInitialAssignment(m->getSpecies(id), 
-                                        m->getInitialAssignment(i)))
-            {
-              delete m->removeInitialAssignment(id);
-              count--;
-            }
-          }
-          else 
-          {
-            for (unsigned int j = 0; j < m->getNumReactions(); j++)
-            {
-              Reaction * r = m->getReaction(j);
-              unsigned int k;
-              for (k = 0; k < r->getNumProducts(); k++)
-              {
-                if (r->getProduct(k)->getId() == id)
-                {
-                  if (expandInitialAssignment(r->getProduct(k), 
-                                              m->getInitialAssignment(i)))
-                  {
-                    delete m->removeInitialAssignment(id);
-                    count--;
-                  }
-                }
-              }
-              for (k = 0; k < r->getNumReactants(); k++)
-              {
-                if (r->getReactant(k)->getId() == id)
-                {
-                  if (expandInitialAssignment(r->getReactant(k), 
-                                              m->getInitialAssignment(i)))
-                  {
-                    delete m->removeInitialAssignment(id);
-                    count--;
-                  }
-                }
-              }
-            }
+            bool removed = expandIA(m, m->getInitialAssignment(i));
+            if (removed) count--;
           }
         }
+        else
+        {
+          needToBail = true;
+        }
       }
       else
       {
-        needToBail = true;
+        count--;
       }
     }
     /* if count is still same nothing changed so bail or endlessly loop */
@@ -1106,7 +1206,72 @@ SBMLTransforms::expandInitialAssignments(Model * m)
       needToBail = true;
     }
   }
-  while(m->getNumInitialAssignments() > 0 && needToBail == false);
+  while(count > 0 && needToBail == false);
+
+  // clear the internal map of values
+  mValues.clear();
+
+  return true;
+}
+
+
+bool 
+SBMLTransforms::expandL3V2InitialAssignments(Model * m)
+{
+  IdList idsNoValues = mapComponentValues(m);
+  IdList idsWithValues;
+
+  IdValueIter iter;
+  bool needToBail = false;
+  unsigned int count;
+
+  do
+  {
+    /* need a fail safe in case a value is just missing */
+    unsigned int num = m->getNumInitialAssignments();
+    count = num;
+    
+    /* list ids that have a calculated/assigned value */
+    idsWithValues.clear();
+    for (iter = mValues.begin(); iter != mValues.end(); ++iter)
+    {
+      if (((*iter).second).second)
+      {
+        idsWithValues.append((*iter).first);
+      }
+    }
+
+    for (int i = num-1; i >= 0; i--)
+    {
+      const InitialAssignment* ia = m->getInitialAssignment(i);
+      // we only want to do something if the initial assignment uses l3v2 math
+      if (ia->isSetMath() && ia->getMath()->usesL3V2MathConstructs() && !ia->getMath()->usesRateOf())
+      {
+        if (!nodeContainsId(ia->getMath(), idsNoValues))
+        {
+          if (!nodeContainsNameNotInList(ia->getMath(), idsWithValues))
+          {
+            bool removed = expandIA(m, ia);
+            if (removed) count--;
+          }
+        }
+        else
+        {
+          needToBail = true;
+        }
+      }
+      else
+      {
+        count--;
+      }
+    }
+    /* if count is still same nothing changed so bail or endlessly loop */
+    if (count == num)
+    {
+      needToBail = true;
+    }
+  }
+  while(count > 0 && needToBail == false);
 
   // clear the internal map of values
   mValues.clear();
