@@ -8,7 +8,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2014 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -198,7 +198,7 @@
  * by AssignmentRule or AlgebraicRule, and in addition, for simulation time
  * <em>t &gt; 0</em>, it may also be changed by a RateRule or Event
  * objects.  (However, some of these constructs are mutually exclusive; see
- * the SBML Level&nbsp;3 Version&nbsp;1 Core specifiation for more
+ * the SBML Level&nbsp;3 Core specifiation for more
  * details.)  It is not an error to define "stoichiometry" on a species
  * reference and also redefine the stoichiometry using an
  * InitialAssignment, but the "stoichiometry" attribute in that case is
@@ -276,10 +276,12 @@ public:
    * Creates a new SpeciesReference using the given SBML @p level and @p version
    * values.
    *
-   * @param level an unsigned int, the SBML Level to assign to this SpeciesReference
+   * @param level an unsigned int, the SBML Level to assign to this SpeciesReference.
    *
    * @param version an unsigned int, the SBML Version to assign to this
-   * SpeciesReference
+   * SpeciesReference.
+   *
+   * @copydetails doc_throw_exception_lv
    *
    * @copydetails doc_note_setting_lv
    */
@@ -292,6 +294,8 @@ public:
    *
    * @param sbmlns an SBMLNamespaces object.
    *
+   * @copydetails doc_throw_exception_namespace
+   *
    * @copydetails doc_note_setting_lv
    */
   SpeciesReference (SBMLNamespaces* sbmlns);
@@ -307,9 +311,6 @@ public:
    * Copy constructor; creates a copy of this SpeciesReference.
    * 
    * @param orig the SpeciesReference instance to copy.
-   *
-   * @throws SBMLConstructorException
-   * Thrown if the argument @p orig is @c NULL.
    */
   SpeciesReference (const SpeciesReference& orig);
 
@@ -317,15 +318,13 @@ public:
   /**
    * Assignment operator
    *
-   * @param rhs The object whose values are used as the basis of the
+   * @param rhs the object whose values are used as the basis of the
    * assignment.
-   *
-   * @throws SBMLConstructorException
-   * Thrown if the argument @p rhs is @c NULL.
    */
   SpeciesReference& operator=(const SpeciesReference& rhs);
 
 
+  /** @cond doxygenLibsbmlInternal */
   /**
    * Accepts the given SBMLVisitor.
    *
@@ -334,6 +333,7 @@ public:
    * @return the result of calling <code>v.visit()</code>.
    */
   virtual bool accept (SBMLVisitor& v) const;
+  /** @endcond */
 
 
   /**
@@ -542,7 +542,7 @@ public:
    * In SBML Level 3, there is no StoichiometryMath, and SpeciesReference
    * objects have only the "stoichiometry" attribute.
    * 
-   * @param value the new value of the "stoichiometry" attribute
+   * @param value the new value of the "stoichiometry" attribute.
    *
    * @note In SBML Level&nbsp;2, the "stoichiometryMath" subelement of this
    * SpeciesReference object will be unset because the "stoichiometry"
@@ -615,7 +615,7 @@ public:
    * "stoichiometry" and "denominator" in the case of SBML Level&nbsp;1, or
    * a "stoichiometryMath" subelement in the case of SBML Level&nbsp;2).
    *
-   * @param value the scalar value 
+   * @param value the scalar value.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -628,7 +628,7 @@ public:
    * @p flag.
    *
    * @param flag a boolean, the value for the "constant" attribute of this
-   * SpeciesReference instance
+   * SpeciesReference instance.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -695,11 +695,21 @@ public:
   int unsetStoichiometry ();
 
 
+  /**
+   * Unsets the "constant" attribute of this SpeciesReference.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_UNEXPECTED_ATTRIBUTE, OperationReturnValues_t}
+   */
+  int unsetConstant ();
+
+
   /**
    * Creates a new, empty StoichiometryMath object, adds it to this
    * SpeciesReference, and returns it.
    *
-   * @return the newly created StoichiometryMath object instance
+   * @return the newly created StoichiometryMath object instance.
    *
    * @see Reaction::addReactant(const SpeciesReference* sr)
    * @see Reaction::addProduct(const SpeciesReference* sr)
@@ -719,9 +729,9 @@ public:
    * discarded.  An alternative may be to use appendAnnotation().
    *
    * @param annotation an XML structure that is to be used as the content
-   * of the "annotation" subelement of this object
+   * of the "annotation" subelement of this object.
    *
-   * @copydetails doc_returns_success_code
+   * @copydetails doc_returns_one_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
    *
    * @see appendAnnotation(const XMLNode* annotation)
@@ -742,7 +752,7 @@ public:
    * discarded.  An alternative may be to use appendAnnotation().
    *
    * @param annotation an XML string that is to be used as the content
-   * of the "annotation" subelement of this object
+   * of the "annotation" subelement of this object.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -764,7 +774,7 @@ public:
    * adds its own data.
    *
    * @param annotation an XML structure that is to be copied and appended
-   * to the content of the "annotation" subelement of this object
+   * to the content of the "annotation" subelement of this object.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -786,7 +796,7 @@ public:
    * adds its own data.
    *
    * @param annotation an XML string that is to be copied and appended
-   * to the content of the "annotation" subelement of this object
+   * to the content of the "annotation" subelement of this object.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -824,7 +834,7 @@ public:
   /** @cond doxygenLibsbmlInternal */
   /**
    * Subclasses should override this method to write out their contained
-   * SBML objects as XML elements.  Be sure to call your parents
+   * SBML objects as XML elements.  Be sure to call your parent's
    * implementation of this method as well.
    */
   virtual void writeElements (XMLOutputStream& stream) const;
@@ -855,9 +865,337 @@ public:
   virtual bool hasRequiredAttributes() const ;
 
 
-protected:
+
+
+
+
+  #ifndef SWIG
+
+
+
   /** @cond doxygenLibsbmlInternal */
 
+  /**
+   * Gets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName, bool& value)
+    const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName, int& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           double& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           unsigned int& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           std::string& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           const char* value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Predicate returning @c true if this SpeciesReference's attribute
+   * "attributeName" is set.
+   *
+   * @param attributeName, the name of the attribute to query.
+   *
+   * @return @c true if this SpeciesReference's attribute "attributeName" has
+   * been set, otherwise @c false is returned.
+   */
+  virtual bool isSetAttribute(const std::string& attributeName) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, bool value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, int value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, double value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName,
+                           unsigned int value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName,
+                           const std::string& value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, const char*
+    value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Unsets the value of the "attributeName" attribute of this
+   * SpeciesReference.
+   *
+   * @param attributeName, the name of the attribute to query.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int unsetAttribute(const std::string& attributeName);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Creates and returns an new "elementName" object in this SpeciesReference.
+   *
+   * @param objectName, the name of the element to create.
+   *
+   * pointer to the object created.
+   */
+  virtual SBase* createChildObject(const std::string& elementName);
+
+  /** @endcond */
+
+  
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Returns the number of "elementName" in this SpeciesReference.
+   *
+   * @param elementName, the name of the element to get number of.
+   *
+   * unsigned int number of elements.
+   */
+  virtual unsigned int getNumObjects(const std::string& elementName);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Returns the nth object of "objectName" in this SpeciesReference.
+   *
+   * @param elementName, the name of the element to get number of.
+   *
+   * @param index, unsigned int teh index of teh object to retrieve.
+   *
+   * pointer to the object.
+   */
+  virtual SBase* getObject(const std::string& elementName, unsigned int index);
+
+  /** @endcond */
+
+
+
+
+  #endif /* !SWIG */
+
+
+protected:
+  /** @cond doxygenLibsbmlInternal */
   /**
    * Create and return a speciesReference object, if present.
    *
@@ -870,7 +1208,7 @@ protected:
    * Subclasses should override this method to read (and store) XHTML,
    * MathML, etc. directly from the XMLInputStream.
    *
-   * @return true if the subclass read from the stream, false otherwise.
+   * @return @c true if the subclass read from the stream, @c false otherwise.
    */
   bool readOtherXML (XMLInputStream& stream);
 
@@ -887,7 +1225,7 @@ protected:
   /**
    * Subclasses should override this method to read values from the given
    * XMLAttributes set into their specific fields.  Be sure to call your
-   * parents implementation of this method as well.
+   * parent's implementation of this method as well.
    */
   virtual void readAttributes (const XMLAttributes& attributes,
                                const ExpectedAttributes& expectedAttributes);
@@ -902,7 +1240,7 @@ protected:
 
   /**
    * Subclasses should override this method to write their XML attributes
-   * to the XMLOutputStream.  Be sure to call your parents implementation
+   * to the XMLOutputStream.  Be sure to call your parent's implementation
    * of this method as well.
    */
   virtual void writeAttributes (XMLOutputStream& stream) const;
@@ -967,9 +1305,13 @@ public:
    * The object is constructed such that it is valid for the given SBML
    * Level and Version combination.
    *
-   * @param level the SBML Level
+   * @param level the SBML Level.
    * 
-   * @param version the Version within the SBML Level
+   * @param version the Version within the SBML Level.
+   *
+   * @copydetails doc_throw_exception_lv
+   *
+   * @copydetails doc_note_setting_lv
    */
   ListOfSpeciesReferences (unsigned int level, unsigned int version);
           
@@ -983,6 +1325,10 @@ public:
    *
    * @param sbmlns an SBMLNamespaces object that is used to determine the
    * characteristics of the ListOfSpeciesReferences object to be created.
+   *
+   * @copydetails doc_throw_exception_namespace
+   *
+   * @copydetails doc_note_setting_lv
    */
   ListOfSpeciesReferences (SBMLNamespaces* sbmlns);
 
@@ -1085,7 +1431,7 @@ public:
    *
    * The caller owns the returned item and is responsible for deleting it.
    *
-   * @param n the index of the item to remove
+   * @param n the index of the item to remove.
    *
    * @see size()
    */
@@ -1099,7 +1445,7 @@ public:
    * If none of the items in this list have the identifier @p sid, then @c
    * NULL is returned.
    *
-   * @param sid the identifier of the item to remove
+   * @param sid the identifier of the item to remove.
    *
    * @return the item removed.  As mentioned above, the caller owns the
    * returned item.
@@ -1108,7 +1454,6 @@ public:
 
 
   /** @cond doxygenLibsbmlInternal */
-
   /**
    * Get the ordinal position of this element in the containing object
    * (which in this case is the Model object).
@@ -1123,7 +1468,6 @@ public:
 
 protected:
   /** @cond doxygenLibsbmlInternal */
-
   enum SpeciesType { Unknown, Reactant, Product, Modifier };
 
 
@@ -1165,10 +1509,10 @@ BEGIN_C_DECLS
  * and @p version values.
  *
  * @param level an unsigned int, the SBML Level to assign to this
- * SpeciesReference_t
+ * SpeciesReference_t.
  *
  * @param version an unsigned int, the SBML Version to assign to this
- * SpeciesReference_t
+ * SpeciesReference_t.
  *
  * @return a pointer to the newly created SpeciesReference_t structure.
  *
@@ -1192,7 +1536,7 @@ SpeciesReference_create (unsigned int level, unsigned int version);
  * SBMLNamespaces_t structure.
  *
  * @param sbmlns SBMLNamespaces_t, a pointer to an SBMLNamespaces_t structure
- * to assign to this SpeciesReference_t
+ * to assign to this SpeciesReference_t.
  *
  * @return a pointer to the newly created SpeciesReference_t structure.
  *
@@ -1216,10 +1560,10 @@ SpeciesReference_createWithNS (SBMLNamespaces_t *sbmlns);
  * using the given SBMLNamespaces_t structure.
  *
  * @param level an unsigned int, the SBML Level to assign to this
- * SpeciesReference_t
+ * SpeciesReference_t.
  *
  * @param version an unsigned int, the SBML Version to assign to this
- * SpeciesReference_t
+ * SpeciesReference_t.
  *
  * @return a pointer to the newly created SpeciesReference_t structure.
  *
@@ -1243,7 +1587,7 @@ SpeciesReference_createModifier (unsigned int level, unsigned int version);
  * using the given SBMLNamespaces_t structure.
  *
  * @param sbmlns SBMLNamespaces_t, a pointer to an SBMLNamespaces_t structure
- * to assign to this modifier SpeciesReference_t
+ * to assign to this modifier SpeciesReference_t.
  *
  * @return a pointer to the newly created SpeciesReference_t structure.
  *
@@ -1265,7 +1609,7 @@ SpeciesReference_createModifierWithNS (SBMLNamespaces_t *sbmlns);
 /**
  * Frees the given SpeciesReference_t structure.
  *
- * @param sr The SpeciesReference_t structure.
+ * @param sr the SpeciesReference_t structure.
  *
  * @memberof SpeciesReference_t
  */
@@ -1278,7 +1622,7 @@ SpeciesReference_free (SpeciesReference_t *sr);
  * Creates and returns a deep copy of the given SpeciesReference_t
  * structure.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return a (deep) copy of this SpeciesReference_t.
  *
@@ -1299,7 +1643,7 @@ SpeciesReference_clone (const SpeciesReference_t *sr);
  * This function has no effect if the SpeciesReference_t structure is a
  * modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
  * @memberof SpeciesReference_t
  */
@@ -1312,10 +1656,10 @@ SpeciesReference_initDefaults (SpeciesReference_t *sr);
  * Returns a list of XMLNamespaces_t associated with this SpeciesReference_t
  * structure.
  *
- * @param sr the SpeciesReference_t structure
+ * @param sr the SpeciesReference_t structure.
  * 
  * @return pointer to the XMLNamespaces_t structure associated with 
- * this structure
+ * this structure.
  *
  * @memberof SpeciesReference_t
  */
@@ -1328,7 +1672,7 @@ SpeciesReference_getNamespaces(SpeciesReference_t *sr);
  * Predicate returning @c true or @c false depending on whether the
  * given SpeciesReference_t structure is a modifier.
  * 
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return nonzero if this SpeciesReference_t represents a modifier
  * species, zero (0)if it is a plain SpeciesReference_t.
@@ -1344,7 +1688,7 @@ SpeciesReference_isModifier (const SpeciesReference_t *sr);
  * Get the value of the "id" attribute of the given SpeciesReference_t
  * structure.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return the identifier of the SpeciesReference_t instance.
  *
@@ -1359,7 +1703,7 @@ SpeciesReference_getId (const SpeciesReference_t *sr);
  * Get the value of the "name" attribute of the given SpeciesReference_t
  * structure.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return the name of the SpeciesReference_t instance.
  *
@@ -1374,9 +1718,9 @@ SpeciesReference_getName (const SpeciesReference_t *sr);
  * Get the value of the "species" attribute of the given SpeciesReference_t
  * structure.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
- * @return the "species" attribute value
+ * @return the "species" attribute value.
  *
  * @memberof SpeciesReference_t
  */
@@ -1392,9 +1736,9 @@ SpeciesReference_getSpecies (const SpeciesReference_t *sr);
  * This function returns zero if the SpeciesReference_t structure is a
  * Modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
- * @return the "stoichiometry" attribute value
+ * @return the "stoichiometry" attribute value.
  *
  * @memberof SpeciesReference_t
  */
@@ -1410,7 +1754,7 @@ SpeciesReference_getStoichiometry (const SpeciesReference_t *sr);
  * This function returns NULL if the SpeciesReference_t structure is a
  * Modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return the stoichiometryMath of this SpeciesReference_t.
  *
@@ -1439,7 +1783,7 @@ SpeciesReference_getStoichiometryMath (SpeciesReference_t *sr);
  * This function returns 0 if the SpeciesReference_t structure is a Modifer (see
  * SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return the denominator of this SpeciesReference_t.
  *
@@ -1453,7 +1797,7 @@ SpeciesReference_getDenominator (const SpeciesReference_t *sr);
 /**
  * Get the value of the "constant" attribute.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return the constant attribute of this SpeciesReference_t.
  *
@@ -1469,7 +1813,7 @@ SpeciesReference_getConstant (const SpeciesReference_t *sr);
  * whether the "id" attribute of the given SpeciesReference_t structure is
  * set.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return nonzero if the "id" attribute of given SpeciesReference_t
  * structure is set, zero (0) otherwise.
@@ -1486,7 +1830,7 @@ SpeciesReference_isSetId (const SpeciesReference_t *sr);
  * whether the "name" attribute of the given SpeciesReference_t
  * structure is set.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return nonzero if the "name" attribute of given SpeciesReference_t
  * structure is set, zero (0) otherwise.
@@ -1503,7 +1847,7 @@ SpeciesReference_isSetName (const SpeciesReference_t *sr);
  * whether the "species" attribute of the given SpeciesReference_t
  * structure is set.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return nonzero if the "species" attribute of given SpeciesReference_t
  * structure is set, zero (0) otherwise.
@@ -1523,7 +1867,7 @@ SpeciesReference_isSetSpecies (const SpeciesReference_t *sr);
  * This function returns false if the SpeciesReference_t structure is a
  * Modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return nonzero if the "stoichiometryMath" subelement has content, zero
  * (0) otherwise.
@@ -1540,7 +1884,7 @@ SpeciesReference_isSetStoichiometryMath (const SpeciesReference_t *sr);
  * whether the "stoichiometry" attribute of the given SpeciesReference_t structure is
  * set.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return nonzero if the "stoichiometry" attribute of given SpeciesReference_t
  * structure is set, zero (0) otherwise.
@@ -1557,7 +1901,7 @@ SpeciesReference_isSetStoichiometry (const SpeciesReference_t *sr);
  * whether the "constant" attribute of the given SpeciesReference_t structure is
  * set.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  * 
  * @return nonzero if the "constant" attribute of given SpeciesReference_t
  * structure is set, zero (0) otherwise.
@@ -1575,9 +1919,9 @@ SpeciesReference_isSetConstant (const SpeciesReference_t *sr);
  *
  * The string in @p sid will be copied.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
- * @param sid The identifier string that will be copied and assigned as the
+ * @param sid the identifier string that will be copied and assigned as the
  * "id" attribute value.
  *
  * @copydetails doc_returns_success_code
@@ -1601,9 +1945,9 @@ SpeciesReference_setId (SpeciesReference_t *sr, const char *sid);
  *
  * The string in @p sid will be copied.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
- * @param name The identifier string that will be copied and assigned as the
+ * @param name the identifier string that will be copied and assigned as the
  * "name" attribute value.
  *
  * @copydetails doc_returns_success_code
@@ -1627,9 +1971,9 @@ SpeciesReference_setName (SpeciesReference_t *sr, const char *name);
  *
  * The string in @p sid will be copied.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
- * @param sid The identifier string that will be copied and assigned as the
+ * @param sid the identifier string that will be copied and assigned as the
  * "species" attribute value.
  *
  * @copydetails doc_returns_success_code
@@ -1653,11 +1997,11 @@ SpeciesReference_setSpecies (SpeciesReference_t *sr, const char *sid);
  * This function has no effect if the SpeciesReference_t structure is a
  * Modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
- * @param value The value to assign to the "stoichiometry" attribute.
+ * @param value the value to assign to the "stoichiometry" attribute.
  *
- * @copydetails doc_returns_success_code
+ * @copydetails doc_returns_one_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
  *
  * @memberof SpeciesReference_t
@@ -1671,7 +2015,7 @@ SpeciesReference_setStoichiometry (SpeciesReference_t *sr, double value);
  * Creates a new, empty StoichiometryMath_t structure, adds it to the
  * @p sr SpeciesReference_t, and returns it.
  *
- * @return the newly created StoichiometryMath_t structure instance
+ * @return the newly created StoichiometryMath_t structure instance.
  *
  * @see Reaction_addReactant()
  * @see Reaction_addProduct()
@@ -1690,9 +2034,9 @@ SpeciesReference_createStoichiometryMath (SpeciesReference_t *sr);
  * This function has no effect if the SpeciesReference_t structure is a
  * Modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
- * @param math The StoichiometryMath_t structure to use in the given SpeciesReference_t.
+ * @param math the StoichiometryMath_t structure to use in the given SpeciesReference_t.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1726,11 +2070,11 @@ SpeciesReference_setStoichiometryMath (  SpeciesReference_t *sr
  * This function has no effect if the SpeciesReference_t structure is a
  * Modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
- * @param value The value to assign to the "denominator" attribute.
+ * @param value the value to assign to the "denominator" attribute.
  *
- * @copydetails doc_returns_success_code
+ * @copydetails doc_returns_one_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
  *
  * @memberof SpeciesReference_t
@@ -1762,7 +2106,7 @@ SpeciesReference_setConstant (SpeciesReference_t *sr, int value);
  * Unsets the value of the "id" attribute of the given SpeciesReference_t
  * structure.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1779,7 +2123,7 @@ SpeciesReference_unsetId (SpeciesReference_t *sr);
  * Unsets the value of the "name" attribute of the given SpeciesReference_t
  * structure.
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1792,6 +2136,40 @@ int
 SpeciesReference_unsetName (SpeciesReference_t *sr);
 
 
+/**
+ * Unsets the value of the "species" attribute of the given SpeciesReference_t
+ * structure.
+ *
+ * @param sr the SpeciesReference_t structure to use.
+ *
+ * @copydetails doc_returns_success_code
+ * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+ * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+ *
+ * @memberof SpeciesReference_t
+ */
+LIBSBML_EXTERN
+int
+SpeciesReference_unsetSpecies (SpeciesReference_t *sr);
+
+
+/**
+ * Unsets the value of the "constant" attribute of the given SpeciesReference_t
+ * structure.
+ *
+ * @param sr the SpeciesReference_t structure to use.
+ *
+ * @copydetails doc_returns_success_code
+ * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+ * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+ *
+ * @memberof SpeciesReference_t
+ */
+LIBSBML_EXTERN
+int
+SpeciesReference_unsetConstant (SpeciesReference_t *sr);
+
+
 /**
  * Unsets the content of the "stoichiometryMath" subelement of the given
  * SpeciesReference_t structure.
@@ -1799,7 +2177,7 @@ SpeciesReference_unsetName (SpeciesReference_t *sr);
  * This function has no effect if the SpeciesReference_t structure is a
  * Modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1819,7 +2197,7 @@ SpeciesReference_unsetStoichiometryMath (SpeciesReference_t *sr);
  * This function has no effect if the SpeciesReference_t structure is a
  * Modifer (see SpeciesReference_isModifier()).
  *
- * @param sr The SpeciesReference_t structure to use.
+ * @param sr the SpeciesReference_t structure to use.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -1844,7 +2222,7 @@ SpeciesReference_unsetStoichiometry (SpeciesReference_t *sr);
   * @param sr the SpeciesReference_t structure to check.
   *
   * @return a true if all the required
-  * attributes for this object have been defined, false otherwise.
+  * attributes for this object have been defined, @c false otherwise.
   *
  * @memberof SpeciesReference_t
  */
@@ -1877,7 +2255,7 @@ ListOfSpeciesReferences_getById (ListOf_t *lo, const char *sid);
  * The caller owns the returned item and is responsible for deleting it.
  *
  * @param lo the list of SpeciesReference_t structures to search.
- * @param sid the "id" attribute value of the structure to remove
+ * @param sid the "id" attribute value of the structure to remove.
  *
  * @return The SpeciesReference_t structure removed, or a null pointer if no such
  * item exists in @p lo.
