@@ -9,12 +9,12 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -90,8 +90,8 @@ struct CloneASTPluginEntityNoParent : public unary_function<ASTBasePlugin*, ASTB
 
 
 ASTBase::ASTBase (int type) :
-   mIsChildFlag     ( false )
-   , mPackageName      ( "core" )
+//   mIsChildFlag     ( false )
+     mPackageName      ( "core" )
    , mId ("")
    , mClass ("")
    , mStyle ("")
@@ -116,8 +116,8 @@ ASTBase::ASTBase (int type) :
   
 
 ASTBase::ASTBase (SBMLNamespaces* sbmlns, int type) :
-   mIsChildFlag     ( false )
-   , mPackageName      ( "core" )
+//   mIsChildFlag     ( false )
+     mPackageName      ( "core" )
    , mId ("")
    , mClass ("")
    , mStyle ("")
@@ -141,8 +141,8 @@ ASTBase::ASTBase (SBMLNamespaces* sbmlns, int type) :
  * Copy constructor
  */
 ASTBase::ASTBase (const ASTBase& orig):
-   mIsChildFlag          ( orig.mIsChildFlag )  
-  , mType                ( orig.mType )
+//   mIsChildFlag          ( orig.mIsChildFlag )  
+    mType                ( orig.mType )
   , mTypeFromPackage     ( orig.mTypeFromPackage)
   , mPackageName         ( orig.mPackageName )
   , mId                  (orig.mId)
@@ -158,7 +158,7 @@ ASTBase::ASTBase (const ASTBase& orig):
              mPlugins.begin(), CloneASTPluginEntity() );
   for (size_t i=0; i < mPlugins.size(); i++)
   {
-    getPlugin(i)->connectToParent(this);
+    getPlugin((unsigned int)i)->connectToParent(this);
   }
 }
 
@@ -171,7 +171,7 @@ ASTBase::operator=(const ASTBase& rhs)
 {
   if(&rhs!=this)
   {
-    mIsChildFlag          = rhs.mIsChildFlag;
+//    mIsChildFlag          = rhs.mIsChildFlag;
     mType                 = rhs.mType;
     mTypeFromPackage      = rhs.mTypeFromPackage;
     mPackageName          = rhs.mPackageName;
@@ -472,7 +472,7 @@ ASTBase::isPackageInfixFunction() const
   if (getType() != AST_ORIGINATES_IN_PACKAGE) return false;
   for (size_t i=0; i < mPlugins.size(); i++)
   {
-    if (getPlugin(i)->isPackageInfixFunction()) return true;
+    if (getPlugin((unsigned int)i)->isPackageInfixFunction()) return true;
   }
   return false;
 }
@@ -483,7 +483,7 @@ ASTBase::hasPackageOnlyInfixSyntax() const
   if (getType() != AST_ORIGINATES_IN_PACKAGE) return false;
   for (size_t i=0; i < mPlugins.size(); i++)
   {
-    if (getPlugin(i)->hasPackageOnlyInfixSyntax()) return true;
+    if (getPlugin((unsigned int)i)->hasPackageOnlyInfixSyntax()) return true;
   }
   return false;
 }
@@ -494,7 +494,7 @@ ASTBase::getL3PackageInfixPrecedence() const
   if (getType() != AST_ORIGINATES_IN_PACKAGE) return 8;
   for (size_t i=0; i < mPlugins.size(); i++)
   {
-    int ret = getPlugin(i)->getL3PackageInfixPrecedence();
+    int ret = getPlugin((unsigned int)i)->getL3PackageInfixPrecedence();
     if (ret != -1) return ret;
   }
   return 8;
@@ -506,7 +506,7 @@ ASTBase::hasUnambiguousPackageInfixGrammar(const ASTNode *child) const
   if (getType() != AST_ORIGINATES_IN_PACKAGE) return false;
   for (size_t i=0; i < mPlugins.size(); i++)
   {
-    if (getPlugin(i)->hasUnambiguousPackageInfixGrammar(child)) return true;
+    if (getPlugin((unsigned int)i)->hasUnambiguousPackageInfixGrammar(child)) return true;
   }
   return false;
 }
@@ -629,6 +629,7 @@ ASTBase::isConstantNumber() const
     case AST_CONSTANT_FALSE:
     case AST_CONSTANT_PI:
     case AST_CONSTANT_TRUE:
+    case AST_NAME_AVOGADRO:
       isNumber = true;
       break;
     default:
@@ -661,6 +662,7 @@ ASTBase::isCSymbolFunction() const
   switch (getType())
   {
   case AST_FUNCTION_DELAY:
+  case AST_FUNCTION_RATE_OF:
     isCsymbolFunc = true;
     break;
   default:
@@ -723,7 +725,12 @@ ASTBase::isFunction() const
   bool isFunction = false;
 
   int type = getType();
-  if (type >= AST_FUNCTION && type <= AST_FUNCTION_TANH)
+  if ((type >= AST_FUNCTION && type <= AST_FUNCTION_TANH)
+    || (type <= AST_FUNCTION_REM && type >= AST_FUNCTION_MAX))
+  {
+    isFunction = true;
+  }
+  else if (type == AST_CSYMBOL_FUNCTION)
   {
     isFunction = true;
   }
@@ -769,6 +776,10 @@ ASTBase::isLogical() const
   {
     isLogical = true;
   }
+  else if (type == AST_LOGICAL_IMPLIES)
+  {
+    isLogical = true;
+  }
   else if (getNumPlugins() > 0)
   {
     unsigned int i = 0;
@@ -1274,12 +1285,12 @@ ASTBase::setIsBvar(bool isbvar)
 
 
 void 
-ASTBase::write(XMLOutputStream& stream) const
+ASTBase::write(XMLOutputStream& ) const
 {
 }
 
 bool 
-ASTBase::read(XMLInputStream& stream, const std::string& reqd_prefix)
+ASTBase::read(XMLInputStream& stream, const std::string& )
 {
   ExpectedAttributes expectedAttributes;
   addExpectedAttributes(expectedAttributes, stream);
@@ -1417,18 +1428,22 @@ ASTBase::readAttributes(const XMLAttributes& attributes,
   return read;
 }
 
-
+#if (0)
+/*
+ * removed as caused issues when manipulating ASTs
+ */
 bool 
 ASTBase::isChild() const
 {
   return mIsChildFlag;
 }
-
+#endif
 
 void 
-ASTBase::setIsChildFlag(bool flag)
+ASTBase::setIsChildFlag(bool )
 {
-  mIsChildFlag = flag;
+  // no op
+  // left for backwards compatibility reasons
 }
 
 //
@@ -1448,7 +1463,7 @@ ASTBase::addPlugin(ASTBasePlugin* plugin)
  * Returns a plugin object (extenstion interface) of package extension
  * with the given package name or URI.
  *
- * @param package the name or URI of the package
+ * @param package the name or URI of the package.
  *
  * @return the plugin object of package extension with the given package
  * name or URI. 
@@ -1482,7 +1497,7 @@ ASTBase::getPlugin(const std::string& package)
  * Returns a plugin object (extenstion interface) of package extension
  * with the given package name or URI.
  *
- * @param package the name or URI of the package
+ * @param package the name or URI of the package.
  *
  * @return the plugin object of package extension with the given package
  * name or URI. 
@@ -1507,7 +1522,7 @@ ASTBase::getPlugin(unsigned int n)
  * Returns a plugin object (extenstion interface) of package extension
  * with the given package name or URI.
  *
- * @param package the name or URI of the package
+ * @param package the name or URI of the package.
  *
  * @return the plugin object of package extension with the given package
  * name or URI. 
@@ -1527,7 +1542,7 @@ ASTBase::getPlugin(unsigned int n) const
 unsigned int 
 ASTBase::getNumPlugins() const
 {
-  return (int)mPlugins.size();
+  return (unsigned int)mPlugins.size();
 }
 
 
@@ -1634,8 +1649,8 @@ ASTBase::writeAttributes (XMLOutputStream& stream) const
 
 
 void 
-ASTBase::writeNodeOfType(XMLOutputStream& stream, int type, 
-    bool inChildNode) const
+ASTBase::writeNodeOfType(XMLOutputStream& , int ,
+    bool ) const
 {
 }
 
@@ -1691,7 +1706,7 @@ ASTBase::loadASTPlugins(const SBMLNamespaces * sbmlns)
   if (sbmlns == NULL)
   {
     const std::vector<std::string>& names = SBMLExtensionRegistry::getAllRegisteredPackageNames();
-    unsigned int numPkgs = names.size();
+    unsigned int numPkgs = (unsigned int)names.size();
     
     for (unsigned int i=0; i < numPkgs; i++)
     {
@@ -1705,10 +1720,11 @@ ASTBase::loadASTPlugins(const SBMLNamespaces * sbmlns)
         const ASTBasePlugin* astPlugin = sbmlext->getASTBasePlugin();
         if (astPlugin != NULL)
         {
-          //// need to give the plugin infomrtaion about itself
-          //astPlugin->setSBMLExtension(sbmlext);
-          //astPlugin->connectToParent(this);
-          mPlugins.push_back(astPlugin->clone());
+            ASTBasePlugin* myastPlugin = astPlugin->clone();
+            myastPlugin->setSBMLExtension(sbmlext);
+//            myastPlugin->setPrefix(xmlns->getPrefix(i));
+            myastPlugin->connectToParent(this);
+            mPlugins.push_back(myastPlugin);
         }
 
       }
@@ -1752,7 +1768,7 @@ ASTBase::syncMembersFrom(ASTBase* rhs)
     return;
   }
 
-  mIsChildFlag          = rhs->mIsChildFlag;
+//  mIsChildFlag          = rhs->mIsChildFlag;
   mType                 = rhs->mType;
   mTypeFromPackage      = rhs->mTypeFromPackage;
   mPackageName          = rhs->mPackageName;
@@ -1779,7 +1795,7 @@ ASTBase::syncPluginsFrom(ASTBase* rhs)
     return;
   }
 
-  mIsChildFlag          = rhs->mIsChildFlag;
+//  mIsChildFlag          = rhs->mIsChildFlag;
   mType                 = rhs->mType;
   mTypeFromPackage      = rhs->mTypeFromPackage;
   mPackageName          = rhs->mPackageName;
@@ -1807,7 +1823,7 @@ ASTBase::syncMembersAndResetParentsFrom(ASTBase* rhs)
     return;
   }
 
-  mIsChildFlag          = rhs->mIsChildFlag;
+//  mIsChildFlag          = rhs->mIsChildFlag;
   mType                 = rhs->mType;
   mTypeFromPackage      = rhs->mTypeFromPackage;
   mPackageName          = rhs->mPackageName;
@@ -1841,7 +1857,7 @@ ASTBase::syncMembersOnlyFrom(ASTBase* rhs)
     return;
   }
 
-  mIsChildFlag          = rhs->mIsChildFlag;
+//  mIsChildFlag          = rhs->mIsChildFlag;
   mId                   = rhs->mId;
   mClass                = rhs->mClass;
   mStyle                = rhs->mStyle;
