@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -67,12 +67,17 @@ public:
  
   /**
    * Copy constructor
+   *
+   * @param orig the instance to copy.
    */
   ASTBase (const ASTBase& orig);
   
 
   /**
    * Assignment operator for ASTNode.
+   *
+   * @param rhs the object whose values are used as the basis of the
+   * assignment.
    */
   ASTBase& operator=(const ASTBase& rhs);
 
@@ -111,7 +116,7 @@ public:
    * of doing this is that any numerical values previously stored in this
    * node are reset to zero.
    *
-   * @param type the type to which this node should be set
+   * @param type the type to which this node should be set.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -205,10 +210,15 @@ public:
   //bool readAttributes(const XMLAttributes att);
 
   /* functions to keep note of whether this instance of an AST is a child */
+  // removed as this could cause issues when manipulating ASTs
+#if (0)
   bool isChild() const;
+#endif
 
-
+  // this is now a no-op but left for backwards compatibility reasons
+  // since it did trickle up to the exposed ASTNode class
   virtual void setIsChildFlag(bool flag);
+  
 
   // functions for MathML attributes
   std::string getClass() const;
@@ -251,7 +261,7 @@ public:
    * Returns a plug-in object (extension interface) for an SBML Level&nbsp;3
    * package extension with the given package name or URI.
    *
-   * @param package the name or URI of the package
+   * @param package the name or URI of the package.
    *
    * @return the plug-in object (the libSBML extension interface) of
    * a package extension with the given package name or URI.
@@ -263,7 +273,7 @@ public:
    * Returns a plug-in object (extension interface) for an SBML Level&nbsp;3
    * package extension with the given package name or URI.
    *
-   * @param package the name or URI of the package
+   * @param package the name or URI of the package.
    *
    * @return the plug-in object (the libSBML extension interface) of a
    * package extension with the given package name or URI.
@@ -275,7 +285,7 @@ public:
    * Returns the nth plug-in object (extension interface) for an SBML Level&nbsp;3
    * package extension.
    *
-   * @param n the index of the plug-in to return
+   * @param n the index of the plug-in to return.
    *
    * @return the plug-in object (the libSBML extension interface) of
    * a package extension with the given package name or URI.
@@ -287,7 +297,7 @@ public:
    * Returns the nth plug-in object (extension interface) for an SBML Level&nbsp;3
    * package extension.
    *
-   * @param n the index of the plug-in to return
+   * @param n the index of the plug-in to return.
    *
    * @return the plug-in object (the libSBML extension interface) of a
    * package extension with the given package name or URI.
@@ -333,14 +343,14 @@ public:
   virtual const std::string& getUnitsPrefix() const;
   
   /**
-   * Returns true if this is a package function which should be written as
-   * "functionname(argumentlist)", false otherwise.
+   * Returns @c true if this is a package function which should be written as
+   * "functionname(argumentlist)", @c false otherwise.
    */
   virtual bool isPackageInfixFunction() const;
 
   /**
-   * Returns true if this is a package function which should be written
-   * special syntax that the package knows about, false otherwise.
+   * Returns @c true if this is a package function which should be written
+   * special syntax that the package knows about, @c false otherwise.
    */
   virtual bool hasPackageOnlyInfixSyntax() const;
 
@@ -350,8 +360,8 @@ public:
   virtual int getL3PackageInfixPrecedence() const;
 
   /**
-   * Returns true if this is a package function which needs no special
-   * consideration when writng as infix, false otherwise.
+   * Returns @c true if this is a package function which needs no special
+   * consideration when writng as infix, @c false otherwise.
    */
   virtual bool hasUnambiguousPackageInfixGrammar(const ASTNode *child) const;
 
@@ -391,7 +401,10 @@ protected:
 
   /* member variables */
 
-  bool mIsChildFlag;
+/*
+ * removed as caused issues when manipulating ASTs
+ */
+//  bool mIsChildFlag;
 
   ASTNodeType_t mType;
   int mTypeFromPackage;
