@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -58,17 +58,12 @@ ASTNaryFunctionNode::ASTNaryFunctionNode (int type) :
 }
   
 
-  /**
-   * Copy constructor
-   */
 ASTNaryFunctionNode::ASTNaryFunctionNode (const ASTNaryFunctionNode& orig):
   ASTFunctionBase(orig)
     , mReducedToBinary (orig.mReducedToBinary)
 {
 }
-  /**
-   * Assignment operator for ASTNode.
-   */
+
 ASTNaryFunctionNode&
 ASTNaryFunctionNode::operator=(const ASTNaryFunctionNode& rhs)
 {
@@ -79,9 +74,7 @@ ASTNaryFunctionNode::operator=(const ASTNaryFunctionNode& rhs)
   }
   return *this;
 }
-  /**
-   * Destroys this ASTNode, including any child nodes.
-   */
+
 ASTNaryFunctionNode::~ASTNaryFunctionNode ()
 {
 }
@@ -448,8 +441,6 @@ ASTNaryFunctionNode::isSqrt() const
 void
 ASTNaryFunctionNode::write(XMLOutputStream& stream) const
 {
-  if (&stream == NULL) return;
-
   int type  = getType();
   unsigned int numChildren = getNumChildren();
 
@@ -761,7 +752,5 @@ ASTNaryFunctionNode::getReducedToBinary() const
   return mReducedToBinary;
 }
 LIBSBML_CPP_NAMESPACE_END
-
-
 /** @endcond */
 
