@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2016 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -411,9 +411,6 @@ ASTBase()
 /** @endcond */
 
 
-/*
-   * Copy constructor
-   */
 ASTNode::ASTNode (const ASTNode& orig) :
         ASTBase(orig)
       , mNumber  ( NULL )
@@ -445,10 +442,6 @@ ASTNode::ASTNode (const ASTNode& orig) :
 }
 
 
-/*
- * 
-   * Assignment operator for ASTNode.
-   */
 ASTNode&
 ASTNode::operator=(const ASTNode& rhs)
 {
@@ -1881,6 +1874,52 @@ ASTNode::freeName()
 }
 
 
+/** @cond doxygenLibsbmlInternal */
+bool
+ASTNode::usesL3V2MathConstructs() const
+{
+  bool mathConstructs = false;
+
+  int type = getType();
+  if (type == AST_FUNCTION_RATE_OF ||
+      type == AST_FUNCTION_MIN ||
+      type == AST_FUNCTION_MAX ||
+      type == AST_FUNCTION_REM ||
+      type == AST_FUNCTION_QUOTIENT ||
+      type == AST_LOGICAL_IMPLIES)
+      mathConstructs = true;
+  unsigned int i = 0;
+  while (!mathConstructs && i < getNumChildren())
+  {
+    mathConstructs = getChild(i)->usesL3V2MathConstructs();
+    i++;
+  }
+
+  return mathConstructs;
+}
+/** @endcond */
+
+
+/** @cond doxygenLibsbmlInternal */
+bool
+ASTNode::usesRateOf() const
+{
+  bool mathConstructs = false;
+
+  if (getType() == AST_FUNCTION_RATE_OF)
+      mathConstructs = true;
+  unsigned int i = 0;
+  while (!mathConstructs && i < getNumChildren())
+  {
+    mathConstructs = getChild(i)->usesRateOf();
+    i++;
+  }
+
+  return mathConstructs;
+}
+/** @endcond */
+
+
 /** @cond doxygenLibsbmlInternal */
 bool
 ASTNode::hasCnUnits() const
@@ -2032,6 +2071,24 @@ ASTNode::isConstant() const
 }
 
 
+bool 
+ASTNode::isConstantNumber() const
+{
+  bool valid = false;
+  
+  if (mNumber != NULL)
+  {
+    valid = mNumber->isConstantNumber();
+  }
+  else if (mFunction != NULL)
+  {
+    valid = false;
+  }
+
+  return valid;
+}
+
+
 bool 
 ASTNode::isFunction() const
 {
@@ -2439,7 +2496,10 @@ ASTNode::returnsBoolean (const Model* givenModel /*=NULL*/) const
 
       if (fd != NULL && fd->isSetMath())
       {
-        return fd->getMath()->getRightChild()->returnsBoolean();
+        // when getting the function from a FD need to use getBody();
+//        return fd->getMath()->getRightChild()->returnsBoolean();
+        return (fd->getBody() != NULL ?
+          fd->getBody()->returnsBoolean() : false);
       }
       else
       {
@@ -2779,6 +2839,22 @@ ASTNode::read(XMLInputStream& stream, const std::string& reqd_prefix)
 
     stream.skipText();
     read = ASTNode::read(stream, reqd_prefix);
+    // we may have a legitimate read but the next token is not the end of math
+    // we want to tell the user but allow the astnode as-is
+    if (read && !(stream.getErrorLog()->contains(BadMathML)))
+    {
+      stream.skipText();
+      const XMLToken element1 = stream.peek();
+      const string&  name = element1.getName();
+      if (element1.isEndFor(element) == false)
+      {
+        std::string message = "Unexpected element encountered. The element <" +
+          name + "> should not be encountered here.";
+        logError(stream, element, InvalidMathElement, message);
+        stream.skipPastEnd(element);
+      }
+
+    }
   }
   else if (isTopLevelMathMLNumberNodeTag(name) == true)
   {
@@ -2897,10 +2973,18 @@ ASTNode::replaceArgument(const std::string& bvar, ASTNode *arg)
       else if (arg->isReal())
       {
         this->setValue(arg->getReal());
+        if (arg->isSetUnits())
+        {
+          this->setUnits(arg->getUnits());
+        }
       }
       else if (arg->isInteger())
       {
         this->setValue(arg->getInteger());
+        if (arg->isSetUnits())
+        {
+          this->setUnits(arg->getUnits());
+        }
       }
       else if (arg->isConstant())
       {
@@ -2933,10 +3017,18 @@ ASTNode::replaceArgument(const std::string& bvar, ASTNode *arg)
         else if (arg->isReal())
         {
           child->setValue(arg->getReal());
+          if (arg->isSetUnits())
+          {
+            child->setUnits(arg->getUnits());
+          }
         }
         else if (arg->isInteger())
         {
           child->setValue(arg->getInteger());
+          if (arg->isSetUnits())
+          {
+            child->setUnits(arg->getUnits());
+          }
         }
         else if (arg->isConstant())
         {
@@ -3416,6 +3508,21 @@ const ASTBasePlugin*
 {
   return const_cast<ASTNode*>(this)->getPlugin(n);
 }
+
+unsigned int
+ASTNode::getNumPiece() const
+{
+  if (getType() == AST_FUNCTION_PIECEWISE)
+  {
+    return static_cast<ASTPiecewiseFunctionNode*>
+      (getFunction()->getPiecewise())->getNumPiece();
+  }
+  else
+  {
+    return 0;
+  }
+}
+
 /** @endcond */
 
 bool
@@ -3776,7 +3883,7 @@ LIBSBML_EXTERN
 int
 ASTNode_addChild (ASTNode_t *node, ASTNode_t *child)
 {
-	if (node == NULL) return LIBSBML_INVALID_OBJECT;
+  if (node == NULL) return LIBSBML_INVALID_OBJECT;
   return static_cast<ASTNode*>(node)->addChild
                                     ( static_cast<ASTNode*>(child) );
 }
@@ -4044,6 +4151,15 @@ ASTNode_isConstant (const ASTNode_t *node)
 }
 
 
+LIBSBML_EXTERN
+int
+ASTNode_isConstantNumber(const ASTNode_t *node)
+{
+  if (node == NULL) return (int)false;
+  return (int) static_cast<const ASTNode*>(node)->isConstantNumber();
+}
+
+
 LIBSBML_EXTERN
 int
 ASTNode_isFunction (const ASTNode_t *node)
