@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -43,14 +43,8 @@ using namespace std;
 
 /* end doxygen comment */
 
-#if defined(_MSC_VER) || defined(__BORLANDC__)
-#  define isnan(d)  _isnan(d)
-#endif
-
 LIBSBML_CPP_NAMESPACE_BEGIN
 
-
-
 ASTNumber::ASTNumber (int type) :
    ASTBase   (type)
   , mExponential    ( NULL )
@@ -109,9 +103,6 @@ ASTNumber::ASTNumber (int type) :
   
 
  
-  /**
-   * Copy constructor
-   */
 ASTNumber::ASTNumber (const ASTNumber& orig):
     ASTBase (orig)
       , mExponential    ( NULL )
@@ -159,9 +150,7 @@ ASTNumber::ASTNumber (const ASTNumber& orig):
                                 ( orig.mCSymbol->deepCopy() );
   }
 }
-  /**
-   * Assignment operator for ASTNode.
-   */
+
 ASTNumber&
 ASTNumber::operator=(const ASTNumber& rhs)
 {
@@ -1729,7 +1718,7 @@ ASTNumber::setValue(double value, long value1)
 int 
 ASTNumber::setValue(double value)
 {
-  if (mExponential == NULL && mReal == NULL && !(isnan(value) > 0 || util_isInf(value) != 0))
+  if (mExponential == NULL && mReal == NULL && !(util_isNaN(value) > 0 || util_isInf(value) != 0))
   {
     std::string units = ASTNumber::getUnits();
     reset();
@@ -1738,7 +1727,7 @@ ASTNumber::setValue(double value)
     setType(AST_REAL);
     mReal->ASTBase::syncMembersFrom(this);
   }
-  else if ((isnan(value) > 0 || util_isInf(value) != 0) && mConstant == NULL) 
+  else if ((util_isNaN(value) > 0 || util_isInf(value) != 0) && mConstant == NULL) 
   {
     reset();
     mConstant = new ASTConstantNumberNode(AST_REAL);
@@ -2960,7 +2949,5 @@ ASTNumber::reset()
 
 
 LIBSBML_CPP_NAMESPACE_END
-
-
 /** @endcond */
 
