@@ -12,7 +12,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2016 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -327,6 +327,20 @@ L3FormulaFormatter_isGrouped (const ASTNode_t *parent, const ASTNode_t *child, c
 
   if (parent != NULL)
   {
+    if (isUnaryMinus(parent)) {
+      child = L3FormulaFormatter_getRightChild(parent);
+      if (isUnaryNot(child)) {
+        //Always group: -(!a) because reasons.
+        return 1;
+      }
+    }
+    else if (isUnaryNot(parent)) {
+      child = L3FormulaFormatter_getRightChild(parent);
+      if (isUnaryMinus(child)) {
+        //Always group: !(-a) because reasons.
+        return 1;
+      }
+    }
     if (ASTNode_isLogical(parent) || ASTNode_isRelational(parent) || isTranslatedModulo(parent)) {
       if (!L3FormulaFormatter_hasUnambiguousGrammar(NULL, child, settings)) {
         //Always group potentially-ambiguous children of logical, relational, and modulo operators, since their precedence is unfamiliar to most users.
@@ -471,7 +485,9 @@ L3FormulaFormatter_formatFunction (StringBuffer_t *sb, const ASTNode_t *node, co
   case AST_FUNCTION_DELAY:
     StringBuffer_append(sb, "delay");
     break;
-
+  case AST_FUNCTION_RATE_OF:
+    StringBuffer_append(sb, "rateOf");
+    break;
   default:
     FormulaFormatter_formatFunction(sb, node);
     break;
@@ -564,8 +580,8 @@ L3FormulaFormatter_formatReal (StringBuffer_t *sb, const ASTNode_t *node, const
   {
     if (ASTNode_getType(node) == AST_REAL_E)
     {
-      StringBuffer_appendExp(sb, value);
-    }
+	    StringBuffer_appendFullExp(sb, ASTNode_getMantissa(node), ASTNode_getExponent(node), value);
+	  }
     else
     {
       StringBuffer_appendReal(sb, value);
@@ -1036,6 +1052,12 @@ L3FormulaFormatter_isFunction (const ASTNode_t *node,
   case AST_FUNCTION_SINH:
   case AST_FUNCTION_TAN:
   case AST_FUNCTION_TANH:
+  case AST_FUNCTION_MAX:
+  case AST_FUNCTION_MIN:
+  case AST_FUNCTION_QUOTIENT:
+  case AST_FUNCTION_RATE_OF:
+  case AST_FUNCTION_REM:
+  case AST_LOGICAL_IMPLIES:
   case AST_UNKNOWN:
     return 1;
 
@@ -1047,5 +1069,7 @@ L3FormulaFormatter_isFunction (const ASTNode_t *node,
   assert(0);
   return 1;
 }
+
+
 /** @endcond */
 
