@@ -8,7 +8,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -40,6 +40,11 @@
 #include <sbml/util/ElementFilter.h>
 #include <sbml/util/PrefixTransformer.h>
 
+#ifdef LIBSBML_HAS_PACKAGE_FBC
+#include <sbml/packages/fbc/extension/FbcModelPlugin.h>
+#endif // LIBSBML_HAS_PACKAGE_FBC
+
+
 using namespace std;
 
 LIBSBML_CPP_NAMESPACE_BEGIN
@@ -544,6 +549,23 @@ Model* CompModelPlugin::flattenModel() const
       delete flat;
       return NULL;
     }
+#ifdef LIBSBML_HAS_PACKAGE_FBC
+    // for an fbc v2 model we need to check that the model element 
+    // in the flat document has the fbc:strict attribute
+    // note this can happen if the parent document did not have fbc but
+    // included a submodel from an external document that did
+    if (SBMLExtensionRegistry::isPackageEnabled("fbc"))
+    {
+      FbcModelPlugin *mplugin = static_cast<FbcModelPlugin*>
+        (flat->getPlugin("fbc"));
+      if (mplugin != NULL && mplugin->getPackageVersion() == 2
+        && mplugin->isSetStrict() == false)
+      {
+        mplugin->setStrict(false);
+      }
+    }
+#endif // LIBSBML_HAS_PACKAGE_FBC
+
   }
 
   // Now we clear the saved referenced elements in the local Port objects, 
@@ -758,12 +780,11 @@ CompModelPlugin::instantiateSubmodels()
 int CompModelPlugin::saveAllReferencedElements()
 {
   set<SBase*> norefs;
-  return saveAllReferencedElements(norefs, norefs);
+  return saveAllReferencedElements(norefs, norefs, getSBMLDocument());
 }
 
-int CompModelPlugin::saveAllReferencedElements(set<SBase*> uniqueRefs, set<SBase*> replacedBys)
+int CompModelPlugin::saveAllReferencedElements(set<SBase*> uniqueRefs, set<SBase*> replacedBys, SBMLDocument* doc)
 {
-  SBMLDocument* doc = getSBMLDocument();
   Model* model = static_cast<Model*>(getParentSBMLObject());
   if (model==NULL) {
     if (doc) {
@@ -915,7 +936,7 @@ int CompModelPlugin::saveAllReferencedElements(set<SBase*> uniqueRefs, set<SBase
     if (subplug==NULL) {
       return LIBSBML_OPERATION_FAILED;
     }
-    ret = subplug->saveAllReferencedElements(uniqueRefs, replacedBys);
+    ret = subplug->saveAllReferencedElements(uniqueRefs, replacedBys, doc);
     if (ret != LIBSBML_OPERATION_SUCCESS) {
       return ret;
     }
@@ -1085,7 +1106,7 @@ void CompModelPlugin::renameIDs(List* allElements, const string& prefix)
   for (unsigned long el=0; el < allElements->getSize(); ++el) 
   {
     SBase* element = static_cast<SBase*>(allElements->get((unsigned int)el));
-    string id = element->getId();
+    string id = element->getIdAttribute();
     string metaid = element->getMetaId();
     
     // if a custom prefix transformer was specified, use it, other wise
@@ -1098,7 +1119,7 @@ void CompModelPlugin::renameIDs(List* allElements, const string& prefix)
     if (element->getTypeCode() == SBML_LOCAL_PARAMETER) {
       element->setId(id); //Change it back.  This would perhaps be better served by overriding 'prependStringToAllIdentifiers' but hey.
     }
-    string newid = element->getId();
+    string newid = element->getIdAttribute();
     string newmetaid = element->getMetaId();
     if (id != newid) {
       int type = element->getTypeCode();
@@ -1297,6 +1318,7 @@ int CompModelPlugin::performReplacementsAndConversions()
 }
 /** @endcond */
 
+/** @cond doxygenLibsbmlInternal */
 int CompModelPlugin::removeCollectedElements(set<SBase*>* removed, set<SBase*>* toremove)
 {
   while (toremove->size() > 0) {
@@ -1315,6 +1337,7 @@ int CompModelPlugin::removeCollectedElements(set<SBase*>* removed, set<SBase*>*
   }
   return LIBSBML_OPERATION_SUCCESS;
 }
+/** @endcond */
 
   
 /** @cond doxygenLibsbmlInternal */
