@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -63,7 +63,7 @@ SubmodelReferenceCycles::~SubmodelReferenceCycles ()
  * event assignments and assignment rules.
  */
 void
-SubmodelReferenceCycles::check_ (const Model& m, const Model& object)
+SubmodelReferenceCycles::check_ (const Model& m, const Model&)
 {
   mIdMap.clear();
 
@@ -282,7 +282,7 @@ SubmodelReferenceCycles::determineCycles(const Model& m)
    
   for (unsigned int n = 0; n < variables.size(); n++)
   {
-    id = variables.at(n);
+    id = variables.at((int)n);
     range = mIdMap.equal_range(id);
     for (it = range.first; it != range.second; it++)
     {
