@@ -1,13 +1,13 @@
 /**
- * @file:   FbcAnd.h
- * @brief:  Implementation of the FbcAnd class
- * @author: SBMLTeam
+ * @file   FbcAnd.h
+ * @brief  Implementation of the FbcAnd class
+ * @author SBMLTeam
  *
  * <!--------------------------------------------------------------------------
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2014 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -29,6 +29,23 @@
  * in the file named "LICENSE.txt" included with this software distribution
  * and also available online as http://sbml.org/software/libsbml/license.html
  * ------------------------------------------------------------------------ -->
+ *
+ * @class FbcAnd
+ * @sbmlbrief{fbc} An &ldquo;and&rdquo; relationship for gene products
+ *
+ * FbcAnd class is used in Version 2 of the SBML Level&nbsp;3 @ref fbc
+ * (&ldquo;fbc&rdquo;) package to represent an "and" relationship between two
+ * or more child FbcAssociation objects.  In other words, it indicates that
+ * all of the child objects are included.  Note that since the FbcAssociation
+ * class is the parent class of GeneProductRef, FbcAnd and FbcOr, a given
+ * FbcAnd can potentially include nested "and"/"or" combinations of gene
+ * products.
+ *
+ * @copydetails doc_note_fbcv2_annotation_replacement
+ *
+ * @see FbcAssociation
+ * @see FbcOr
+ * @see GeneProductRef
  */
 
 
@@ -63,19 +80,26 @@ class LIBSBML_EXTERN FbcAnd : public FbcAssociation
 
 protected:
 
+  /** @cond doxygenLibsbmlInternal */
   ListOfFbcAssociations   mAssociations;
+  /** @endcond */
 
 
 public:
 
   /**
-   * Creates a new FbcAnd with the given level, version, and package version.
+   * Creates a new FbcAnd with the given SBML Level, Version, and
+   * &ldquo;fbc&rdquo;package version.
    *
-   * @param level an unsigned int, the SBML Level to assign to this FbcAnd
+   * @param level an unsigned int, the SBML Level to assign to this FbcAnd.
    *
-   * @param version an unsigned int, the SBML Version to assign to this FbcAnd
+   * @param version an unsigned int, the SBML Version to assign to this
+   * FbcAnd.
    *
-   * @param pkgVersion an unsigned int, the SBML Fbc Version to assign to this FbcAnd
+   * @param pkgVersion an unsigned int, the SBML Fbc Version to assign to
+   * this FbcAnd.
+   *
+   * @copydetails doc_note_setting_lv_pkg
    */
   FbcAnd(unsigned int level      = FbcExtension::getDefaultLevel(),
          unsigned int version    = FbcExtension::getDefaultVersion(),
@@ -85,7 +109,11 @@ public:
   /**
    * Creates a new FbcAnd with the given FbcPkgNamespaces object.
    *
-   * @param fbcns the FbcPkgNamespaces object
+   * @copydetails doc_what_are_sbml_package_namespaces
+   *
+   * @param fbcns the FbcPkgNamespaces object.
+   *
+   * @copydetails doc_note_setting_lv_pkg
    */
   FbcAnd(FbcPkgNamespaces* fbcns);
 
@@ -93,7 +121,7 @@ public:
    /**
    * Copy constructor for FbcAnd.
    *
-   * @param orig; the FbcAnd instance to copy.
+   * @param orig the FbcAnd instance to copy.
    */
   FbcAnd(const FbcAnd& orig);
 
@@ -101,8 +129,8 @@ public:
    /**
    * Assignment operator for FbcAnd.
    *
-   * @param rhs; the object whose values are used as the basis
-   * of the assignment
+   * @param rhs the object whose values are used as the basis
+   * of the assignment.
    */
   FbcAnd& operator=(const FbcAnd& rhs);
 
@@ -138,11 +166,12 @@ public:
 
 
   /**
-   * Get a Association from the ListOfFbcAssociations.
+   * Get the nth object from the ListOfFbcAssociations.
    *
-   * @param n the index number of the Association to get.
+   * @param n the index number of the FbcAssociation to get.
    *
-   * @return the nth Association in the ListOfFbcAssociations within this FbcAnd.
+   * @return the nth FbcAssociation in the ListOfFbcAssociations within this
+   * FbcAnd.
    *
    * @see getNumAssociations()
    */
@@ -150,11 +179,12 @@ public:
 
 
   /**
-   * Get a Association from the ListOfFbcAssociations.
+   * Get the nth object from the ListOfFbcAssociations.
    *
-   * @param n the index number of the Association to get.
+   * @param n the index number of the FbcAssociation to get.
    *
-   * @return the nth Association in the ListOfFbcAssociations within this FbcAnd.
+   * @return the nth FbcAssociation in the ListOfFbcAssociations within this
+   * FbcAnd.
    *
    * @see getNumAssociations()
    */
@@ -162,15 +192,15 @@ public:
 
 
   /**
-   * Get a Association from the ListOfFbcAssociations
+   * Get an FbcAssociation from the ListOfFbcAssociations
    * based on its identifier.
    *
    * @param sid a string representing the identifier
-   * of the Association to get.
+   * of the FbcAssociation to get.
    *
-   * @return the Association in the ListOfFbcAssociations
+   * @return the FbcAssociation in the ListOfFbcAssociations
    * with the given id or NULL if no such
-   * Association exists.
+   * FbcAssociation exists.
    *
    * @see getAssociation(unsigned int n)
    *
@@ -180,15 +210,15 @@ public:
 
 
   /**
-   * Get a Association from the ListOfFbcAssociations
+   * Get an FbcAssociation from the ListOfFbcAssociations
    * based on its identifier.
    *
    * @param sid a string representing the identifier
-   * of the Association to get.
+   * of the FbcAssociation to get.
    *
-   * @return the Association in the ListOfFbcAssociations
+   * @return the FbcAssociation in the ListOfFbcAssociations
    * with the given id or NULL if no such
-   * Association exists.
+   * FbcAssociation exists.
    *
    * @see getAssociation(unsigned int n)
    *
@@ -198,16 +228,18 @@ public:
 
 
   /**
-   * Adds a copy the given "FbcAssociation" to this FbcAnd.
+   * Adds a copy the given FbcAssociation to this FbcAnd.
    *
-   * @param fa; the FbcAssociation object to add
+   * @param fa the FbcAssociation object to add.
    *
-   * @return integer value indicating success/failure of the
-   * function.  @if clike The value is drawn from the
-   * enumeration #OperationReturnValues_t. @endif The possible values
-   * returned by this function are:
-   * @li LIBSBML_OPERATION_SUCCESS
-   * @li LIBSBML_INVALID_ATTRIBUTE_VALUE
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_INVALID_OBJECT, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_LEVEL_MISMATCH, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_VERSION_MISMATCH, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_PKG_VERSION_MISMATCH, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_NAMESPACES_MISMATCH, OperationReturnValues_t}
    */
   int addAssociation(const FbcAssociation* fa);
 
@@ -215,7 +247,7 @@ public:
   /**
    * Get the number of FbcAssociation objects in this FbcAnd.
    *
-   * @return the number of FbcAssociation objects in this FbcAnd
+   * @return the number of FbcAssociation objects in this FbcAnd.
    */
   unsigned int getNumAssociations() const;
 
@@ -225,48 +257,49 @@ public:
   *
   * @return the association as infix string.
   */
-  virtual std::string toInfix() const;
+  virtual std::string toInfix(bool usingId=false) const;
+
 
   /**
-   * Creates a new FbcAnd object, adds it to this FbcAnds
+   * Creates a new FbcAnd object, adds it to this FbcAnd's
    * ListOfFbcAssociations and returns the FbcAnd object created. 
    *
-   * @return a new FbcAnd object instance
+   * @return a new FbcAnd object instance.
    *
-   * @see addFbcAssociation(const FbcAssociation* fa)
+   * @see addAssociation(const FbcAssociation* fa)
    */
   FbcAnd* createAnd();
 
 
   /**
-   * Creates a new FbcOr object, adds it to this FbcAnds
+   * Creates a new FbcOr object, adds it to this FbcAnd's
    * ListOfFbcAssociations and returns the FbcOr object created. 
    *
-   * @return a new FbcOr object instance
+   * @return a new FbcOr object instance.
    *
-   * @see addFbcAssociation(const FbcAssociation* fa)
+   * @see addAssociation(const FbcAssociation* fa)
    */
   FbcOr* createOr();
 
 
   /**
-   * Creates a new GeneProductRef object, adds it to this FbcAnds
+   * Creates a new GeneProductRef object, adds it to this FbcAnd's
    * ListOfFbcAssociations and returns the GeneProductRef object created. 
    *
-   * @return a new GeneProductRef object instance
+   * @return a new GeneProductRef object instance.
    *
-   * @see addFbcAssociation(const FbcAssociation* fa)
+   * @see addAssociation(const FbcAssociation* fa)
    */
   GeneProductRef* createGeneProductRef();
 
 
   /**
-   * Removes the nth Association from the ListOfFbcAssociations within this FbcAnd.
-   * and returns a pointer to it.
+   * Removes the nth FbcAssociation from the ListOfFbcAssociations within this 
+   * FbcAnd and returns a pointer to it.
    *
    * The caller owns the returned item and is responsible for deleting it.
    *
-   * @param n the index of the Association to remove.
+   * @param n the index of the FbcAssociation to remove.
    *
    * @see getNumAssociations()
    */
@@ -274,16 +307,16 @@ public:
 
 
   /**
-   * Removes the Association with the given identifier from the ListOfFbcAssociations within this FbcAnd
-   * and returns a pointer to it.
+   * Removes the FbcAssociation with the given identifier from the 
+   * ListOfFbcAssociations within this FbcAnd and returns a pointer to it.
    *
    * The caller owns the returned item and is responsible for deleting it.
    * If none of the items in this list have the identifier @p sid, then
    * @c NULL is returned.
    *
-   * @param sid the identifier of the Association to remove.
+   * @param sid the identifier of the FbcAssociation to remove.
    *
-   * @return the Association removed. As mentioned above, the caller owns the
+   * @return the FbcAssociation removed. As mentioned above, the caller owns the
    * returned item.
    */
   FbcAssociation* removeAssociation(const std::string& sid);
@@ -293,14 +326,15 @@ public:
    * Returns a List of all child SBase objects, including those nested to an
    * arbitrary depth.
    *
-   * @return a List* of pointers to all child objects.
+   * @return a List of pointers to all child objects.
    */
    virtual List* getAllElements(ElementFilter * filter = NULL);
 
 
   /**
-   * Returns the XML element name of this object, which for FbcAnd, is
-   * always @c "fbcAnd".
+   * Returns the XML element name of this object.
+   *
+   * For FbcAnd, the XML element name is always @c "fbcAnd".
    *
    * @return the name of this element, i.e. @c "fbcAnd".
    */
@@ -309,42 +343,24 @@ public:
 
   /**
    * Returns the libSBML type code for this SBML object.
-   * 
-   * @if clike LibSBML attaches an identifying code to every kind of SBML
-   * object.  These are known as <em>SBML type codes</em>.  The set of
-   * possible type codes is defined in the enumeration #SBMLTypeCode_t.
-   * The names of the type codes all begin with the characters @c
-   * SBML_. @endif@if java LibSBML attaches an identifying code to every
-   * kind of SBML object.  These are known as <em>SBML type codes</em>.  In
-   * other languages, the set of type codes is stored in an enumeration; in
-   * the Java language interface for libSBML, the type codes are defined as
-   * static integer constants in the interface class {@link
-   * libsbmlConstants}.  The names of the type codes all begin with the
-   * characters @c SBML_. @endif@if python LibSBML attaches an identifying
-   * code to every kind of SBML object.  These are known as <em>SBML type
-   * codes</em>.  In the Python language interface for libSBML, the type
-   * codes are defined as static integer constants in the interface class
-   * @link libsbml@endlink.  The names of the type codes all begin with the
-   * characters @c SBML_. @endif@if csharp LibSBML attaches an identifying
-   * code to every kind of SBML object.  These are known as <em>SBML type
-   * codes</em>.  In the C# language interface for libSBML, the type codes
-   * are defined as static integer constants in the interface class @link
-   * libsbmlcs.libsbml@endlink.  The names of the type codes all begin with
-   * the characters @c SBML_. @endif
    *
-   * @return the SBML type code for this object, or
-   * @link SBMLTypeCode_t#SBML_UNKNOWN SBML_UNKNOWN@endlink (default).
+   * @copydetails doc_what_are_typecodes
+   *
+   * @return the SBML type code for this object:
+   * @sbmlconstant{SBML_FBC_AND, SBMLTypeCode_t} (default).
+   *
+   * @copydetails doc_warning_typecodes_not_unique
    *
    * @see getElementName()
+   * @see getPackageName()
    */
   virtual int getTypeCode () const;
 
 
   /**
    * Predicate returning @c true if all the required attributes
-   * for this FbcAnd object have been set.
-   *
-   * @note The required attributes for a FbcAnd object are:
+   * for this FbcAnd object have been set.  Will always return
+   * @c true, since FbcOr has no required attributes.
    *
    * @return a boolean value indicating whether all the required
    * attributes for this object have been defined.
@@ -356,7 +372,8 @@ public:
    * Predicate returning @c true if all the required elements
    * for this FbcAnd object have been set.
    *
-   * @note The required elements for a FbcAnd object are:
+   * @note An FbcAnd object must have at least two FbcAssociation
+   * children.
    *
    * @return a boolean value indicating whether all the required
    * elements for this object have been defined.
@@ -368,13 +385,13 @@ public:
 
   /**
    * Subclasses should override this method to write out their contained
-   * SBML objects as XML elements.  Be sure to call your parents
+   * SBML objects as XML elements.  Be sure to call your parent's
    * implementation of this method as well.
    */
   virtual void writeElements (XMLOutputStream& stream) const;
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -385,7 +402,7 @@ public:
   virtual bool accept (SBMLVisitor& v) const;
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -396,7 +413,7 @@ public:
   virtual void setSBMLDocument (SBMLDocument* d);
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -407,7 +424,7 @@ public:
   virtual void connectToChild ();
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -419,9 +436,334 @@ public:
                const std::string& pkgPrefix, bool flag);
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
+
+  #ifndef SWIG
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName, bool& value)
+    const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName, int& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           double& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           unsigned int& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           std::string& value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Gets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to retrieve.
+   *
+   * @param value, the address of the value to record.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int getAttribute(const std::string& attributeName,
+                           const char* value) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Predicate returning @c true if this FbcAnd's attribute "attributeName" is
+   * set.
+   *
+   * @param attributeName, the name of the attribute to query.
+   *
+   * @return @c true if this FbcAnd's attribute "attributeName" has been set,
+   * otherwise @c false is returned.
+   */
+  virtual bool isSetAttribute(const std::string& attributeName) const;
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, bool value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, int value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, double value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName,
+                           unsigned int value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName,
+                           const std::string& value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Sets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to set.
+   *
+   * @param value, the value of the attribute to set.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int setAttribute(const std::string& attributeName, const char*
+    value);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Unsets the value of the "attributeName" attribute of this FbcAnd.
+   *
+   * @param attributeName, the name of the attribute to query.
+   *
+   * @copydetails doc_returns_success_code
+   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+   */
+  virtual int unsetAttribute(const std::string& attributeName);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Creates and returns an new "elementName" object in this FbcAnd.
+   *
+   * @param elementName, the name of the element to create.
+   *
+   * pointer to the element created.
+   */
+  virtual SBase* createChildObject(const std::string& elementName);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Returns the number of "elementName" in this FbcAnd.
+   *
+   * @param elementName, the name of the element to get number of.
+   *
+   * unsigned int number of elements.
+   */
+  virtual unsigned int getNumObjects(const std::string& elementName);
+
+  /** @endcond */
+
+
+
+  /** @cond doxygenLibsbmlInternal */
+
+  /**
+   * Returns the nth object of "objectName" in this FbcAnd.
+   *
+   * @param elementName, the name of the element to get number of.
+   *
+   * @param index, unsigned int teh index of teh object to retrieve.
+   *
+   * pointer to the object.
+   */
+  virtual SBase* getObject(const std::string& elementName, unsigned int index);
+
+  /** @endcond */
+
+
+
+
+  #endif /* !SWIG */
+
 protected:
 
   /** @cond doxygenLibsbmlInternal */
@@ -432,7 +774,7 @@ protected:
   virtual SBase* createObject(XMLInputStream& stream);
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -443,7 +785,7 @@ protected:
   virtual void addExpectedAttributes(ExpectedAttributes& attributes);
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -455,7 +797,7 @@ protected:
                                const ExpectedAttributes& expectedAttributes);
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -466,7 +808,7 @@ protected:
   virtual void writeAttributes (XMLOutputStream& stream) const;
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
 
