@@ -7,7 +7,7 @@
  * This file is part of libSBML. Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2016 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  * 1. California Institute of Technology, Pasadena, CA, USA
  * 2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  * 3. University of Heidelberg, Heidelberg, Germany
@@ -31,10 +31,14 @@
  * ------------------------------------------------------------------------ -->
  *
  * @class GroupsExtension
- * @sbmlbrief{groups} Base extension class.
+ * @sbmlbrief{groups} Base extension class for the package.
+ *
+ * This is the Groups package extension of the SBMLExtension class that
+ * every libSBML plug-in must implement in order to implement an
+ * SBML Level&nbsp;3 package.
  *
  * @class GroupsPkgNamespaces
- * @sbmlbrief{groups} SBMLNamespaces extension.
+ * @sbmlbrief{groups} SBMLNamespaces extension for the package.
  */
 
 
@@ -131,7 +135,7 @@ public:
   /**
    * Copy constructor for GroupsExtension.
    *
-   * @param orig; the GroupsExtension instance to copy.
+   * @param orig the GroupsExtension instance to copy.
    */
   GroupsExtension(const GroupsExtension& orig);
 
@@ -139,8 +143,8 @@ public:
   /**
    * Assignment operator for GroupsExtension.
    *
-   * @param rhs; the GroupsExtension object whose values are to be used as the
-   * basis of the assignment
+   * @param rhs the GroupsExtension object whose values are to be used as the
+   * basis of the assignment.
    */
   GroupsExtension& operator=(const GroupsExtension& rhs);
 
@@ -171,19 +175,19 @@ public:
    * Returns a string representing the SBML XML namespace of this SBML
    * Level&nbsp;3 package.
    *
-   * @param sbmlLevel, the level of SBML
-   *
-   * @param sbmlVersion the version of SBML
-   *
-   * @param pkgVersion the version of this package
-   *
-   * @return a string representing the name of this package ("groups").
-   *
    * The namespace URI constructed by this method corresponds to the
    * combination of the Level and Version of SBML, and the Version of the SBML
    * Level&nbsp;3 package. (At the time of this writing, the only SBML Level
-   * that supports packages is Level&nbsp;3, so the value of @p sbmlLevel is
-   * necessarily always <code>3</code>.)
+   * that supports packages is Level&nbsp;3, so the value of @p sbmlLevel must
+   * necessarily always be <code>3</code>.)
+   *
+   * @param sbmlLevel the level of SBML.
+   *
+   * @param sbmlVersion the version of SBML.
+   *
+   * @param pkgVersion the version of this package.
+   *
+   * @return a string representing the name of this package ("groups").
    */
   virtual const std::string& getURI(unsigned int sbmlLevel,
                                     unsigned int sbmlVersion,
@@ -194,10 +198,10 @@ public:
    * Returns the SBML Level for the given URI of this package.
    *
    * @param uri the string of the URI that represents one of the versions of
-   * the "groups" package
+   * the "groups" package.
    *
    * @return the SBML Level for the given URI of this package, or @c 0 if the
-   * given URI is invalid.
+   * given URI is invalid, or for a different package.
    */
   virtual unsigned int getLevel(const std::string& uri) const;
 
@@ -207,10 +211,10 @@ public:
    * package.
    *
    * @param uri the string of the URI that represents one of the versions of
-   * the "groups" package
+   * the "groups" package.
    *
    * @return the SBML Version within the SBML Level for the given URI of this
-   * package, or @c 0 if the given URI is invalid.
+   * package, or @c 0 if the given URI is invalid, or for a different package.
    */
   virtual unsigned int getVersion(const std::string& uri) const;
 
@@ -220,10 +224,10 @@ public:
    * package.
    *
    * @param uri the string of the URI that represents one of the versions of
-   * the "groups" package
+   * the "groups" package.
    *
    * @return the version of the SBML Level&nbsp;3 package for the given URI of
-   * this package, or @c 0 if the given URI is invalid.
+   * this package, or @c 0 if the given URI is invalid, or for a different package.
    */
   virtual unsigned int getPackageVersion(const std::string& uri) const;
 
@@ -232,7 +236,7 @@ public:
    * Returns a GroupsPkgNamespaces object.
    *
    * @param uri the string of the URI that represents one of the versions of
-   * the "groups" package
+   * the "groups" package.
    *
    * @return GroupsPkgNamespaces object corresponding to the given URI of this
    * package, or @c NULL if the given URI is not defined in the "groups"
@@ -243,8 +247,8 @@ public:
 
 
   /**
-   * Takes a type code of the &ldquo;groups&rdquo; package and returns a string
-   * describing the code.
+   * Returns a string describing the type code of the &ldquo;groups&rdquo;
+   * package.
    *
    * @param typeCode a libSBML type code defined by the libSBML extension
    * implementing support for the SBML Level&nbsp;3 &ldquo;groups&rdquo;
@@ -366,12 +370,20 @@ typedef enum
 } SBMLGroupsTypeCode_t;
 
 
+/**
+ * @enum GroupKind_t
+ * @brief Enumeration of values permitted as the value of the "kind"
+ * attribute on Group objects.
+ *
+ * @see Group::getKind()
+ * @see Group::setKind(@if java long kind@endif)
+ */
 typedef enum
 {
-  GROUP_KIND_CLASSIFICATION       /*!<classification */
-, GROUP_KIND_PARTONOMY            /*!<partonomy */
-, GROUP_KIND_COLLECTION           /*!<collection */
-, GROUP_KIND_UNKNOWN              /*!<invalid GroupKind */
+  GROUP_KIND_CLASSIFICATION       /*!< The group kind is @c "classification". */
+, GROUP_KIND_PARTONOMY            /*!< The group kind is @c "partonomy". */
+, GROUP_KIND_COLLECTION           /*!< The group kind is @c "collection". */
+, GROUP_KIND_UNKNOWN              /*!< Invalid GroupKind value. */
 } GroupKind_t;
 
 
