@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -35,6 +35,18 @@
  *
  * @class LayoutPkgNamespaces
  * @sbmlbrief{layout} SBMLNamespaces extension for the package.
+ *
+ * There is currently one possible namespace defined for the Layout
+ * package: 
+ * "http://www.sbml.org/sbml/level3/version1/layout/version1".  Despite 
+ * referencing SBML Level&nbsp;3 Version&nbsp;1 explicitly, this package 
+ * (and all such packages) can be used without change in SBML 
+ * Level&nbsp;3 Version&nbsp;2 documents.  The only caveat is that features of 
+ * the SBML Level&nbsp;3 Version&nbsp;2 specification that were not present in 
+ * Level&nbsp;1 may not be used by constructs from the Qualitative Modeling
+ * package.  The main restriction this implies is that the GeneralGlyph may
+ * not reference any SBML element by its "id" attribute if that element would
+ * not have had an "id" attribute in Level&nbsp;3 Version&nbsp;1.
  */
 
 #ifndef LayoutExtension_h
@@ -153,15 +165,17 @@ public:
 
 
   /**
-   * Constructor
+   * Creates a new LayoutExtension instance.
    */
   LayoutExtension ();
 
 
   /**
    * Copy constructor.
+   *
+   * @param orig the instance to copy.
    */
-  LayoutExtension(const LayoutExtension&);
+  LayoutExtension(const LayoutExtension& orig);
 
 
   /**
@@ -172,14 +186,17 @@ public:
 
   /**
    * Assignment operator for LayoutExtension.
+   *
+   * @param orig the object whose values are used as the basis of the
+   * assignment.
    */
-  LayoutExtension& operator=(const LayoutExtension&);
+  LayoutExtension& operator=(const LayoutExtension& orig);
 
 
   /**
    * Creates and returns a deep copy of this LayoutExtension object.
    * 
-   * @return a (deep) copy of this LayoutExtension object
+   * @return a (deep) copy of this LayoutExtension object.
    */
   virtual LayoutExtension* clone () const;
 
@@ -187,18 +204,24 @@ public:
   /**
    * Returns the name of this package ("layout")
    *
-   * @return the name of this package ("layout")
+   * @return the name of this package ("layout").
    */
   virtual const std::string& getName() const;
 
 
   /**
-   * Returns the namespace URI corresponding to the combination of the given
-   * SBML Level, Version, and package version.
+   * Returns a string representing the SBML XML namespace of this SBML
+   * Level&nbsp;3 package.
    *
-   * @param sbmlLevel the level of SBML
-   * @param sbmlVersion the version of SBML
-   * @param pkgVersion the version of package
+   * The namespace URI constructed by this method corresponds to the
+   * combination of the Level and Version of SBML, and the Version of the SBML
+   * Level&nbsp;3 package. (At the time of this writing, the only SBML Level
+   * that supports packages is Level&nbsp;3, so the value of @p sbmlLevel must
+   * necessarily always be <code>3</code>.)
+   *
+   * @param sbmlLevel the level of SBML.
+   * @param sbmlVersion the version of SBML.
+   * @param pkgVersion the version of package.
    *
    * @return a string of the package URI, or an empty string if no
    * corresponding URI exists.
@@ -210,35 +233,35 @@ public:
   /**
    * Returns the SBML Level for the given URI of this package.
    *
-   * @param uri the string of URI that represents one of versions of the
-   * &ldquo;layout&rdquo; package
+   * @param uri a URI that represents a version of this package.
    *
-   * @return the SBML Level with the given URI of this package, or @c 0 if
-   * the given URI is invalid.
+   * @return the SBML Level for the given URI of this package, or @c 0 if the
+   * given URI is invalid, or for a different package.
    */
   virtual unsigned int getLevel(const std::string &uri) const;
 
 
   /**
-   * Returns the SBML Version for the given URI of this package.
+   * Returns the Version within the SBML Level for the given URI of this
+   * package.
    *
-   * @param uri the string of URI that represents one of versions of the
-   * &ldquo;layout&rdquo; package
+   * @param uri a URI that represents a version of this package.
    *
-   * @return the SBML version with the given URI of this package, or @c 0 if
-   * the given URI is invalid.
+   * @return the SBML Version within the SBML Level for the given URI of this
+   * package, or @c 0 if the given URI is invalid, or for a different package.
    */
   virtual unsigned int getVersion(const std::string &uri) const;
 
 
   /**
-   * Returns the package version for the given URI of this package.
+   * Returns the SBML Level&nbsp;3 package version for the given URI of this
+   * package.
    *
-   * @param uri the string of URI that represents one of versions of the
-   * &ldquo;layout&rdquo; package
+   * @param uri a URI that represents one of the valid versions of this
+   * package.
    *
-   * @return the package version with the given URI of this package, or @c 0
-   * if the given URI is invalid.
+   * @return the version of the SBML Level&nbsp;3 package with the given URI,
+   * or @c 0 if the given URI is invalid, or for a different package.
    */
   virtual unsigned int getPackageVersion(const std::string &uri) const;
 
@@ -246,6 +269,14 @@ public:
   /**
    * Takes a type code of the &ldquo;layout&rdquo; package and returns a string
    * describing the code.
+   *
+   * @param typeCode a libSBML type code defined by the libSBML extension.
+   * implementing support for the SBML Level&nbsp;3 &ldquo;layout&rdquo; package.
+   *
+   * @return a text string representing the type code given by @p typeCode.
+   * If the type code is unrecognized for this implementation of the libSBML
+   * &ldquo;layout&rdquo; package, the string returned will be
+   * <code>"(Unknown SBML Layout Type)"</code>.
    */
   virtual const char* getStringFromTypeCode(int typeCode) const;
 
@@ -253,8 +284,8 @@ public:
   /**
    * Returns an LayoutPkgNamespaces object.
    *
-   * @param uri the string of URI that represents one of versions of the
-   * &ldquo;layout&rdquo; package
+   * @param uri a URI that represents one of the valid versions of the
+   * &ldquo;layout&rdquo; package.
    *
    * @return an LayoutPkgNamespace object corresponding to the given @p uri,
    * or @c NULL if the URI is not defined in the Layout package.
@@ -309,10 +340,10 @@ public:
    * The implementation returns true if the model object contains one 
    * or more layouts.
    * 
-   * @param doc the sbml document to test. 
+   * @param doc the sbml document to test.
    * 
    * @return a boolean indicating whether the extension is actually being used
-   *         byy the document. 
+   *         by the document. 
    */
   virtual bool isInUse(SBMLDocument *doc) const;
 
@@ -320,7 +351,7 @@ public:
   /**
    * Return the entry in the error table at this index. 
    *
-   * @param index an unsigned intgere representing the index of the error in the LayoutSBMLErrorTable
+   * @param index an unsigned intgere representing the index of the error in the LayoutSBMLErrorTable.
    *
    * @return packageErrorTableEntry object in the LayoutSBMLErrorTable corresponding to the index given.
    */
@@ -334,7 +365,7 @@ public:
   /**
    * Return the index in the error table with the given errorId. 
    *
-   * @param errorId an unsigned intgere representing the errorId of the error in the LayoutSBMLErrorTable
+   * @param errorId an unsigned intgere representing the errorId of the error in the LayoutSBMLErrorTable.
    *
    * @return unsigned integer representing the index in the LayoutSBMLErrorTable corresponding to the errorId given.
    */
@@ -380,7 +411,8 @@ typedef SBMLExtensionNamespaces<LayoutExtension> LayoutPkgNamespaces;
 
 /**
  * @enum  SBMLLayoutTypeCode_t
- * @brief SBMLLayoutTypeCode_t is the enumeration of possible types from the 'layout' package.
+ * @brief SBMLFbcTypeCode_t Enumeration of possible types in the libSBML
+ * &ldquo;layout&rdquo; package implementation.
  *
  * @copydetails doc_what_are_typecodes
  *
