@@ -139,6 +139,7 @@ const char * SBML_MULTI_TYPECODE_STRINGS[] =
   , "MultiSpeciesType"
   , "BindingSiteSpeciesType"
   , "IntraSpeciesReaction"
+  , "SubListOfSpeciesFeatures"
 };
 
 
@@ -220,7 +221,7 @@ MultiExtension::getURI(unsigned int sbmlLevel,
 {
   if (sbmlLevel == 3)
   {
-    if (sbmlVersion == 1)
+    if (sbmlVersion == 1 || sbmlVersion == 2)
     {
       if (pkgVersion == 1)
       {
@@ -303,7 +304,7 @@ const char*
 MultiExtension::getStringFromTypeCode(int typeCode) const
 {
   int min = SBML_MULTI_POSSIBLE_SPECIES_FEATURE_VALUE;
-  int max = SBML_MULTI_INTRA_SPECIES_REACTION;
+  int max = SBML_MULTI_SUBLIST_OF_SPECIES_FEATURES;
 
   if ( typeCode < min || typeCode > max)
   {
