@@ -28,6 +28,7 @@
 
 
 #include <sbml/packages/multi/sbml/MultiSpeciesType.h>
+#include <sbml/packages/multi/sbml/BindingSiteSpeciesType.h>
 #include <sbml/packages/multi/validator/MultiSBMLError.h>
 
 #include <sbml/util/ElementFilter.h>
@@ -44,8 +45,8 @@ LIBSBML_CPP_NAMESPACE_BEGIN
  */
 MultiSpeciesType::MultiSpeciesType (unsigned int level, unsigned int version, unsigned int pkgVersion)
   : SBase(level, version)
-   ,mId ("")
-   ,mName ("")
+////   ,mId ("")
+////   ,mName ("")
    ,mCompartment ("")
    ,mListOfSpeciesFeatureTypes (level, version, pkgVersion)
    ,mListOfSpeciesTypeInstances (level, version, pkgVersion)
@@ -65,8 +66,8 @@ MultiSpeciesType::MultiSpeciesType (unsigned int level, unsigned int version, un
  */
 MultiSpeciesType::MultiSpeciesType (MultiPkgNamespaces* multins)
   : SBase(multins)
-   ,mId ("")
-   ,mName ("")
+////   ,mId ("")
+////   ,mName ("")
    ,mCompartment ("")
    ,mListOfSpeciesFeatureTypes (multins)
    ,mListOfSpeciesTypeInstances (multins)
@@ -89,8 +90,8 @@ MultiSpeciesType::MultiSpeciesType (MultiPkgNamespaces* multins)
  */
 MultiSpeciesType::MultiSpeciesType (const MultiSpeciesType& orig)
   : SBase(orig)
-  , mId  ( orig.mId)
-  , mName  ( orig.mName)
+//  , mId  ( orig.mId)
+//  , mName  ( orig.mName)
   , mCompartment  ( orig.mCompartment)
   ,mListOfSpeciesFeatureTypes (orig.mListOfSpeciesFeatureTypes)
   ,mListOfSpeciesTypeInstances (orig.mListOfSpeciesTypeInstances)
@@ -942,6 +943,7 @@ MultiSpeciesType::createInSpeciesTypeBond()
 void
 MultiSpeciesType::renameSIdRefs(const std::string& oldid, const std::string& newid)
 {
+  SBase::renameSIdRefs(oldid, newid);
   if (isSetCompartment() == true && mCompartment == oldid)
   {
     setCompartment(newid);
@@ -1389,7 +1391,7 @@ MultiSpeciesType::readAttributes (const XMLAttributes& attributes,
   else
   {
     std::string message = "Multi attribute 'id' is missing.";
-    getErrorLog()->logPackageError("multi", MultiUnknownError,
+    getErrorLog()->logPackageError("multi", MultiSpt_AllowedMultiAtts,
                    getPackageVersion(), sbmlLevel, sbmlVersion, message);
   }
 
@@ -1621,6 +1623,13 @@ ListOfMultiSpeciesTypes::createObject(XMLInputStream& stream)
     appendAndOwn(object);
     delete multins;
   }
+  else if (name == "bindingSiteSpeciesType")
+  {
+    MULTI_CREATE_NS(multins, getSBMLNamespaces());
+    object = new BindingSiteSpeciesType(multins);
+    appendAndOwn(object);
+    delete multins;
+  }
 
   return object;
 }
