@@ -111,6 +111,26 @@ OutwardBindingSite::~OutwardBindingSite ()
 }
 
 
+/*
+ * Returns the value of the "id" attribute of this OutwardBindingSite.
+ */
+const std::string&
+OutwardBindingSite::getId() const
+{
+  return mId;
+}
+
+
+/*
+ * Returns the value of the "name" attribute of this OutwardBindingSite.
+ */
+const std::string&
+OutwardBindingSite::getName() const
+{
+  return mName;
+}
+
+
 /*
  * Returns the value of the "bindingStatus" attribute of this OutwardBindingSite.
  */
@@ -131,6 +151,26 @@ OutwardBindingSite::getComponent() const
 }
 
 
+/*
+ * Returns true/false if id is set.
+ */
+bool
+OutwardBindingSite::isSetId() const
+{
+  return (mId.empty() == false);
+}
+
+
+/*
+ * Returns true/false if name is set.
+ */
+bool
+OutwardBindingSite::isSetName() const
+{
+  return (mName.empty() == false);
+}
+
+
 /*
  * Returns true/false if bindingStatus is set.
  */
@@ -151,6 +191,27 @@ OutwardBindingSite::isSetComponent() const
 }
 
 
+/*
+ * Sets id and returns value indicating success.
+ */
+int
+OutwardBindingSite::setId(const std::string& id)
+{
+  return SyntaxChecker::checkAndSetSId(id, mId);
+}
+
+
+/*
+ * Sets name and returns value indicating success.
+ */
+int
+OutwardBindingSite::setName(const std::string& name)
+{
+  mName = name;
+  return LIBSBML_OPERATION_SUCCESS;
+}
+
+
 /*
  * Sets bindingStatus and returns value indicating success.
  */
@@ -188,6 +249,44 @@ OutwardBindingSite::setComponent(const std::string& component)
 }
 
 
+/*
+ * Unsets id and returns value indicating success.
+ */
+int
+OutwardBindingSite::unsetId()
+{
+  mId.erase();
+
+  if (mId.empty() == true)
+  {
+    return LIBSBML_OPERATION_SUCCESS;
+  }
+  else
+  {
+    return LIBSBML_OPERATION_FAILED;
+  }
+}
+
+
+/*
+ * Unsets name and returns value indicating success.
+ */
+int
+OutwardBindingSite::unsetName()
+{
+  mName.erase();
+
+  if (mName.empty() == true)
+  {
+    return LIBSBML_OPERATION_SUCCESS;
+  }
+  else
+  {
+    return LIBSBML_OPERATION_FAILED;
+  }
+}
+
+
 /*
  * Unsets bindingStatus and returns value indicating success.
  */
@@ -224,6 +323,7 @@ OutwardBindingSite::unsetComponent()
 void
 OutwardBindingSite::renameSIdRefs(const std::string& oldid, const std::string& newid)
 {
+  SBase::renameSIdRefs(oldid, newid);
   if (isSetComponent() == true && mComponent == oldid)
   {
     setComponent(newid);
@@ -344,6 +444,8 @@ OutwardBindingSite::addExpectedAttributes(ExpectedAttributes& attributes)
 {
   SBase::addExpectedAttributes(attributes);
 
+  attributes.add("id");
+  attributes.add("name");
   attributes.add("bindingStatus");
   attributes.add("component");
 }
@@ -431,6 +533,44 @@ OutwardBindingSite::readAttributes (const XMLAttributes& attributes,
 
   bool assigned = false;
 
+  //
+  // id SId  ( use = "optional" )
+  //
+  assigned = attributes.readInto("id", mId);
+
+   if (assigned == true)
+  {
+    // check string is not empty and correct syntax
+
+    if (mId.empty() == true)
+    {
+      logEmptyString(mId, getLevel(), getVersion(), "<OutwardBindingSite>");
+    }
+    else if (SyntaxChecker::isValidSBMLSId(mId) == false && getErrorLog() != NULL)
+    {
+        std::string details = "The syntax of the attribute id='" + mId + "' does not conform.";
+        getErrorLog()->logPackageError("multi", MultiInvSIdSyn,
+                   getPackageVersion(), sbmlLevel, sbmlVersion, details,
+                   getLine(), getColumn());
+    }
+  }
+
+  //
+  // name string   ( use = "optional" )
+  //
+  assigned = attributes.readInto("name", mName);
+
+  if (assigned == true)
+  {
+    // check string is not empty
+
+    if (mName.empty() == true)
+    {
+      logEmptyString(mName, getLevel(), getVersion(), "<OutwardBindingSite>");
+    }
+  }
+
+
   //
   // component SIdRef   ( use = "required" )
   //
@@ -455,7 +595,7 @@ OutwardBindingSite::readAttributes (const XMLAttributes& attributes,
   else
   {
     std::string message = "Multi attribute 'component' is missing.";
-    getErrorLog()->logPackageError("multi", MultiUnknownError,
+    getErrorLog()->logPackageError("multi", MultiOutBst_AllowedMultiAtts,
                    getPackageVersion(), sbmlLevel, sbmlVersion, message,
                    getLine(), getColumn());
   }
@@ -753,6 +893,34 @@ OutwardBindingSite_clone(OutwardBindingSite_t * obs)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+char *
+OutwardBindingSite_getId(OutwardBindingSite_t * cr)
+{
+  if (cr == NULL)
+    return NULL;
+
+  return cr->getId().empty() ? NULL : safe_strdup(cr->getId().c_str());
+}
+
+
+/**
+ *
+ */
+LIBSBML_EXTERN
+char *
+OutwardBindingSite_getName(OutwardBindingSite_t * cr)
+{
+  if (cr == NULL)
+    return NULL;
+
+  return cr->getName().empty() ? NULL : safe_strdup(cr->getName().c_str());
+}
+
+
 /**
  * 
  */
@@ -781,6 +949,28 @@ OutwardBindingSite_getComponent(OutwardBindingSite_t * obs)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+OutwardBindingSite_isSetId(OutwardBindingSite_t * cr)
+{
+  return (cr != NULL) ? static_cast<int>(cr->isSetId()) : 0;
+}
+
+
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+OutwardBindingSite_isSetName(OutwardBindingSite_t * cr)
+{
+  return (cr != NULL) ? static_cast<int>(cr->isSetName()) : 0;
+}
+
+
 /**
  * 
  */
@@ -803,6 +993,28 @@ OutwardBindingSite_isSetComponent(OutwardBindingSite_t * obs)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+OutwardBindingSite_setId(OutwardBindingSite_t * cr, const char * id)
+{
+  return (cr != NULL) ? cr->setId(id) : LIBSBML_INVALID_OBJECT;
+}
+
+
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+OutwardBindingSite_setName(OutwardBindingSite_t * cr, const char * name)
+{
+  return (cr != NULL) ? cr->setName(name) : LIBSBML_INVALID_OBJECT;
+}
+
+
 /**
  * 
  */
@@ -825,6 +1037,28 @@ OutwardBindingSite_setComponent(OutwardBindingSite_t * obs, const char * compone
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+OutwardBindingSite_unsetId(OutwardBindingSite_t * cr)
+{
+  return (cr != NULL) ? cr->unsetId() : LIBSBML_INVALID_OBJECT;
+}
+
+
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+OutwardBindingSite_unsetName(OutwardBindingSite_t * cr)
+{
+  return (cr != NULL) ? cr->unsetName() : LIBSBML_INVALID_OBJECT;
+}
+
+
 /**
  * 
  */
