@@ -122,6 +122,98 @@ public:
   virtual ~OutwardBindingSite();
 
 
+   /**
+   * Returns the value of the "id" attribute of this OutwardBindingSite.
+   *
+   * @return the value of the "id" attribute of this OutwardBindingSite as a string.
+   */
+  virtual const std::string& getId() const;
+
+
+  /**
+   * Predicate returning @c true or @c false depending on whether this
+   * OutwardBindingSite's "id" attribute has been set.
+   *
+   * @return @c true if this OutwardBindingSite's "id" attribute has been set,
+   * otherwise @c false is returned.
+   */
+  virtual bool isSetId() const;
+
+
+  /**
+   * Sets the value of the "id" attribute of this OutwardBindingSite.
+   *
+   * @param id; const std::string& value of the "id" attribute to be set
+   *
+   * @return integer value indicating success/failure of the
+   * function.  @if clike The value is drawn from the
+   * enumeration #OperationReturnValues_t. @endif The possible values
+   * returned by this function are:
+   * @li LIBSBML_OPERATION_SUCCESS
+   * @li LIBSBML_INVALID_ATTRIBUTE_VALUE
+   */
+  virtual int setId(const std::string& id);
+
+
+  /**
+   * Unsets the value of the "id" attribute of this OutwardBindingSite.
+   *
+   * @return integer value indicating success/failure of the
+   * function.  @if clike The value is drawn from the
+   * enumeration #OperationReturnValues_t. @endif The possible values
+   * returned by this function are:
+   * @li LIBSBML_OPERATION_SUCCESS
+   * @li LIBSBML_OPERATION_FAILED
+   */
+  virtual int unsetId();
+
+
+  /**
+   * Returns the value of the "name" attribute of this OutwardBindingSite.
+   *
+   * @return the value of the "name" attribute of this OutwardBindingSite as a string.
+   */
+  virtual const std::string& getName() const;
+
+
+  /**
+   * Predicate returning @c true or @c false depending on whether this
+   * OutwardBindingSite's "name" attribute has been set.
+   *
+   * @return @c true if this OutwardBindingSite's "name" attribute has been set,
+   * otherwise @c false is returned.
+   */
+  virtual bool isSetName() const;
+
+
+  /**
+   * Sets the value of the "name" attribute of this OutwardBindingSite.
+   *
+   * @param name; const std::string& value of the "name" attribute to be set
+   *
+   * @return integer value indicating success/failure of the
+   * function.  @if clike The value is drawn from the
+   * enumeration #OperationReturnValues_t. @endif The possible values
+   * returned by this function are:
+   * @li LIBSBML_OPERATION_SUCCESS
+   * @li LIBSBML_INVALID_ATTRIBUTE_VALUE
+   */
+  virtual int setName(const std::string& name);
+
+
+  /**
+   * Unsets the value of the "name" attribute of this OutwardBindingSite.
+   *
+   * @return integer value indicating success/failure of the
+   * function.  @if clike The value is drawn from the
+   * enumeration #OperationReturnValues_t. @endif The possible values
+   * returned by this function are:
+   * @li LIBSBML_OPERATION_SUCCESS
+   * @li LIBSBML_OPERATION_FAILED
+   */
+  virtual int unsetName();
+
+
    /**
    * Returns the value of the "bindingStatus" attribute of this OutwardBindingSite.
    *
@@ -636,6 +728,16 @@ OutwardBindingSite_t *
 OutwardBindingSite_clone(OutwardBindingSite_t * obs);
 
 
+LIBSBML_EXTERN
+char *
+OutwardBindingSite_getId(OutwardBindingSite_t * cr);
+
+
+LIBSBML_EXTERN
+char *
+OutwardBindingSite_getName(OutwardBindingSite_t * cr);
+
+
 LIBSBML_EXTERN
 BindingStatus_t
 OutwardBindingSite_getBindingStatus(OutwardBindingSite_t * obs);
@@ -646,6 +748,16 @@ char *
 OutwardBindingSite_getComponent(OutwardBindingSite_t * obs);
 
 
+LIBSBML_EXTERN
+int
+OutwardBindingSite_isSetId(OutwardBindingSite_t * cr);
+
+
+LIBSBML_EXTERN
+int
+OutwardBindingSite_isSetName(OutwardBindingSite_t * cr);
+
+
 LIBSBML_EXTERN
 int
 OutwardBindingSite_isSetBindingStatus(OutwardBindingSite_t * obs);
@@ -658,15 +770,34 @@ OutwardBindingSite_isSetComponent(OutwardBindingSite_t * obs);
 
 LIBSBML_EXTERN
 int
-OutwardBindingSite_setBindingStatus(OutwardBindingSite_t * obs, 
-                                    BindingStatus_t bindingStatus);
+OutwardBindingSite_setId(OutwardBindingSite_t * cr, const char * id);
 
 
+LIBSBML_EXTERN
+int
+OutwardBindingSite_setName(OutwardBindingSite_t * cr, const char * name);
+
+
+LIBSBML_EXTERN
+int
+OutwardBindingSite_setBindingStatus(OutwardBindingSite_t * obs, 
+                                    BindingStatus_t bindingStatus);
+
 LIBSBML_EXTERN
 int
 OutwardBindingSite_setComponent(OutwardBindingSite_t * obs, const char * component);
 
 
+LIBSBML_EXTERN
+int
+OutwardBindingSite_unsetId(OutwardBindingSite_t * cr);
+
+
+LIBSBML_EXTERN
+int
+OutwardBindingSite_unsetName(OutwardBindingSite_t * cr);
+
+
 LIBSBML_EXTERN
 int
 OutwardBindingSite_unsetBindingStatus(OutwardBindingSite_t * obs);
