@@ -28,6 +28,7 @@
 
 
 #include <sbml/packages/multi/sbml/SpeciesFeature.h>
+#include <sbml/packages/multi/sbml/SubListOfSpeciesFeatures.h>
 #include <sbml/packages/multi/validator/MultiSBMLError.h>
 
 #include <sbml/util/ElementFilter.h>
@@ -44,7 +45,7 @@ LIBSBML_CPP_NAMESPACE_BEGIN
  */
 SpeciesFeature::SpeciesFeature (unsigned int level, unsigned int version, unsigned int pkgVersion)
   : SBase(level, version)
-   ,mId ("")
+////   ,mId ("")
    ,mSpeciesFeatureType ("")
    ,mOccur (SBML_INT_MAX)
    ,mIsSetOccur (false)
@@ -64,7 +65,7 @@ SpeciesFeature::SpeciesFeature (unsigned int level, unsigned int version, unsign
  */
 SpeciesFeature::SpeciesFeature (MultiPkgNamespaces* multins)
   : SBase(multins)
-   ,mId ("")
+////   ,mId ("")
    ,mSpeciesFeatureType ("")
    ,mOccur (SBML_INT_MAX)
    ,mIsSetOccur (false)
@@ -87,7 +88,7 @@ SpeciesFeature::SpeciesFeature (MultiPkgNamespaces* multins)
  */
 SpeciesFeature::SpeciesFeature (const SpeciesFeature& orig)
   : SBase(orig)
-  , mId  ( orig.mId)
+//  , mId  ( orig.mId)
   , mSpeciesFeatureType  ( orig.mSpeciesFeatureType)
   , mOccur  ( orig.mOccur)
   , mIsSetOccur  ( orig.mIsSetOccur)
@@ -148,6 +149,16 @@ SpeciesFeature::getId() const
 }
 
 
+/*
+ * Returns the value of the "name" attribute of this SpeciesFeature.
+ */
+const std::string&
+SpeciesFeature::getName() const
+{
+  return mName;
+}
+
+
 /*
  * Returns the value of the "speciesFeatureType" attribute of this SpeciesFeature.
  */
@@ -188,6 +199,16 @@ SpeciesFeature::isSetId() const
 }
 
 
+/*
+ * Returns true/false if name is set.
+ */
+bool
+SpeciesFeature::isSetName() const
+{
+  return (mName.empty() == false);
+}
+
+
 /*
  * Returns true/false if speciesFeatureType is set.
  */
@@ -228,6 +249,17 @@ SpeciesFeature::setId(const std::string& id)
 }
 
 
+/*
+ * Sets name and returns value indicating success.
+ */
+int
+SpeciesFeature::setName(const std::string& name)
+{
+  mName = name;
+  return LIBSBML_OPERATION_SUCCESS;
+}
+
+
 /*
  * Sets speciesFeatureType and returns value indicating success.
  */
@@ -295,6 +327,25 @@ SpeciesFeature::unsetId()
 }
 
 
+/*
+ * Unsets name and returns value indicating success.
+ */
+int
+SpeciesFeature::unsetName()
+{
+  mName.erase();
+
+  if (mName.empty() == true)
+  {
+    return LIBSBML_OPERATION_SUCCESS;
+  }
+  else
+  {
+    return LIBSBML_OPERATION_FAILED;
+  }
+}
+
+
 /*
  * Unsets speciesFeatureType and returns value indicating success.
  */
@@ -705,6 +756,17 @@ SpeciesFeature::createObject(XMLInputStream& stream)
 
   if (name == "listOfSpeciesFeatureValues")
   {
+    if (mSpeciesFeatureValues.size() != 0)
+    {
+      getErrorLog()->logPackageError("multi", MultiSpeFtr_RestrictElts,
+        getPackageVersion(), getLevel(), getVersion(), 
+        "<" + getPrefix() + "speciesType> may only have one <" + getPrefix()
+        + "listOfSpeciesFeatureTypes>",
+        stream.peek().getLine(),
+        stream.peek().getColumn());
+
+    }
+
     object = &mSpeciesFeatureValues;
   }
 
@@ -727,6 +789,7 @@ SpeciesFeature::addExpectedAttributes(ExpectedAttributes& attributes)
   SBase::addExpectedAttributes(attributes);
 
   attributes.add("id");
+  attributes.add("name");
   attributes.add("speciesFeatureType");
   attributes.add("occur");
   attributes.add("component");
@@ -756,9 +819,9 @@ SpeciesFeature::readAttributes (const XMLAttributes& attributes,
   */
 
   ListOfSpeciesFeatures * parentListOf =
-      static_cast<ListOfSpeciesFeatures*>(getParentSBMLObject());
+      dynamic_cast<ListOfSpeciesFeatures*>(getParentSBMLObject());
 
-  if (getErrorLog() != NULL && parentListOf->size() < 2)
+  if (getErrorLog() != NULL && parentListOf != NULL && parentListOf->size() < 2)
   {
     numErrs = getErrorLog()->getNumErrors();
     for (int n = numErrs-1; n >= 0; n--)
@@ -768,7 +831,7 @@ SpeciesFeature::readAttributes (const XMLAttributes& attributes,
         const std::string details =
               getErrorLog()->getError(n)->getMessage();
         getErrorLog()->remove(UnknownPackageAttribute);
-        getErrorLog()->logPackageError("multi", MultiLofSpeFtrs_AllowedMultiAtts,
+        getErrorLog()->logPackageError("multi", MultiLofSpeFtrs_AllowedAtts,
                   getPackageVersion(), sbmlLevel, sbmlVersion, details,
                   parentListOf->getLine(), parentListOf->getColumn());
       }
@@ -777,7 +840,7 @@ SpeciesFeature::readAttributes (const XMLAttributes& attributes,
         const std::string details =
                    getErrorLog()->getError(n)->getMessage();
         getErrorLog()->remove(UnknownCoreAttribute);
-        getErrorLog()->logPackageError("multi", MultiLofSpeFtrs_AllowedCoreAtts,
+        getErrorLog()->logPackageError("multi", MultiLofSpeFtrs_AllowedAtts,
                   getPackageVersion(), sbmlLevel, sbmlVersion, details,
                   parentListOf->getLine(), parentListOf->getColumn());
       }
@@ -801,7 +864,8 @@ SpeciesFeature::readAttributes (const XMLAttributes& attributes,
                        getPackageVersion(), sbmlLevel, sbmlVersion, details,
                        getLine(), getColumn());
       }
-      else if (getErrorLog()->getError(n)->getErrorId() == UnknownCoreAttribute)
+      else
+      if (getErrorLog()->getError(n)->getErrorId() == UnknownCoreAttribute)
       {
         const std::string details =
                           getErrorLog()->getError(n)->getMessage();
@@ -837,6 +901,23 @@ SpeciesFeature::readAttributes (const XMLAttributes& attributes,
     }
   }
 
+
+   //
+   // name string   ( use = "optional" )
+   //
+   assigned = attributes.readInto("name", mName);
+
+   if (assigned == true)
+   {
+     // check string is not empty
+
+     if (mName.empty() == true)
+     {
+       logEmptyString(mName, getLevel(), getVersion(), "<SpeciesFeature>");
+     }
+   }
+
+
   //
   // speciesFeatureType SIdRef   ( use = "required" )
   //
@@ -956,9 +1037,6 @@ ListOfSpeciesFeatures::ListOfSpeciesFeatures(unsigned int level,
                         unsigned int version, 
                         unsigned int pkgVersion)
  : ListOf(level, version)
- , mSubListOfSpeciesFeatures (NULL)
- , mRelation (MULTI_RELATION_UNKNOWN)
- , mIsSubList ( false)
 {
   mSubListOfSpeciesFeatures = new List();
 
@@ -973,15 +1051,44 @@ ListOfSpeciesFeatures::ListOfSpeciesFeatures(unsigned int level,
  */
 ListOfSpeciesFeatures::ListOfSpeciesFeatures(MultiPkgNamespaces* multins)
   : ListOf(multins)
-  , mSubListOfSpeciesFeatures (NULL)
-  , mRelation (MULTI_RELATION_UNKNOWN )
-  , mIsSubList (false )
 {
   mSubListOfSpeciesFeatures = new List();
   setElementNamespace(multins->getURI());
   connectToChild();
 }
 
+/*
+ * Copy constructor for ListOfSpeciesFeatures.
+ */
+ListOfSpeciesFeatures::ListOfSpeciesFeatures (const ListOfSpeciesFeatures & orig)
+  : ListOf(orig)
+{
+  mSubListOfSpeciesFeatures = new List();
+  setElementNamespace(orig.getURI());
+  for (unsigned int i = 0; i < orig.getNumSubListOfSpeciesFeatures(); i++) {
+      SubListOfSpeciesFeatures * sub = orig.getSubListOfSpeciesFeatures(i)->clone();
+      addSubListOfSpeciesFeatures(sub);
+  }
+  connectToChild();
+}
+
+/*
+ * Destructor 
+ */
+ListOfSpeciesFeatures::~ListOfSpeciesFeatures()
+{
+  if (mSubListOfSpeciesFeatures != NULL)
+  {
+    unsigned int size = mSubListOfSpeciesFeatures->getSize();
+    while (size > 0) 
+    {
+      delete static_cast<SubListOfSpeciesFeatures*>( mSubListOfSpeciesFeatures->remove(0) );
+      size--;
+    }
+    delete mSubListOfSpeciesFeatures;
+  }
+}
+
 
 /*
  * Returns a deep copy of this ListOfSpeciesFeatures 
@@ -1067,8 +1174,129 @@ ListOfSpeciesFeatures::remove(const std::string& sid)
   return static_cast <SpeciesFeature*> (item);
 }
 
+unsigned int
+ListOfSpeciesFeatures::getNumSpeciesFeatures() const
+{
+  return static_cast<unsigned int>( mItems.size() );
+}
+
+unsigned int
+ListOfSpeciesFeatures::size() const
+{
+  return getNumSpeciesFeatures() + getNumSubListOfSpeciesFeatures();
+}
+
+/*
+ * Get a SubListOfSpeciesFeatures from the ListOfSpeciesFeatures by index.
+*/
+SubListOfSpeciesFeatures*
+ListOfSpeciesFeatures::getSubListOfSpeciesFeatures(unsigned int n)
+{
+  return static_cast<SubListOfSpeciesFeatures*>(mSubListOfSpeciesFeatures->get(n));
+}
+
+
+/*
+ * Get a SubListOfSpeciesFeatures from the ListOfSpeciesFeatures by index.
+ */
+const SubListOfSpeciesFeatures*
+ListOfSpeciesFeatures::getSubListOfSpeciesFeatures(unsigned int n) const
+{
+  return static_cast<const SubListOfSpeciesFeatures*>(mSubListOfSpeciesFeatures->get(n));
+}
+
+
+/*
+ * Get a SubListOfSpeciesFeatures from the ListOfSpeciesFeatures by id.
+ */
+SubListOfSpeciesFeatures*
+ListOfSpeciesFeatures::getSubListOfSpeciesFeatures(const std::string& sid)
+{
+  return const_cast<SubListOfSpeciesFeatures*>(
+    static_cast<const ListOfSpeciesFeatures&>(*this).getSubListOfSpeciesFeatures(sid));
+}
+
+
+/*
+ * Get a SubListOfSpeciesFeatures from the ListOfSpeciesFeatures by id.
+ */
+const SubListOfSpeciesFeatures*
+ListOfSpeciesFeatures::getSubListOfSpeciesFeatures(const std::string& sid) const
+{
+//  vector<SubListOfSpeciesFeatures*>::const_iterator result;
+
+//  result = find_if( mSubListOfSpeciesFeatures->begin(), mSubListOfSpeciesFeatures->end(), IdEq<SubListOfSpeciesFeatures>(sid) );
+//  return (result == mSubListOfSpeciesFeatures->end()) ? 0 : static_cast <SubListOfSpeciesFeatures*> (*result);
+
+  const SubListOfSpeciesFeatures * result = NULL;
+
+
+  if (!sid.empty() && mSubListOfSpeciesFeatures != NULL) {
+      for (unsigned int i = 0; result != NULL  && i < mSubListOfSpeciesFeatures->getSize(); i++) {
+  const SubListOfSpeciesFeatures * item = static_cast <SubListOfSpeciesFeatures*> (mSubListOfSpeciesFeatures->get(i));
+  if (item->getId() == sid) {
+      result = item;
+  }
+      }
+  }
+
+  return result;
+}
+
+
+/*
+ * Removes the nth SubListOfSpeciesFeatures from this ListOfSpeciesFeatures
+ */
+SubListOfSpeciesFeatures*
+ListOfSpeciesFeatures::removeSubListOfSpeciesFeatures(unsigned int n)
+{
+  if (!mSubListOfSpeciesFeatures) {
+      return NULL;
+  }
+
+  return static_cast<SubListOfSpeciesFeatures*>(mSubListOfSpeciesFeatures->remove(n));
+}
+
+
+/*
+ * Removes the SubListOfSpeciesFeatures from this ListOfSpeciesFeatures with the given identifier
+ */
+SubListOfSpeciesFeatures*
+ListOfSpeciesFeatures::removeSubListOfSpeciesFeatures(const std::string& sid)
+{
+//  SBase* item = NULL;
+//  vector<SBase*>::iterator result;
+//
+//  result = find_if((*mSubListOfSpeciesFeatures).begin(), mItems.end(), IdEq<SpeciesFeature>(sid) );
+//
+//  if (result != mItems.end())
+//  {
+//    item = *result;
+//    mItems.erase(result);
+//  }
+
+  SubListOfSpeciesFeatures * result = NULL;
+
+
+  if (!sid.empty() && mSubListOfSpeciesFeatures != NULL) {
+      unsigned int i;
+      for (i = 0; result != NULL && i < mSubListOfSpeciesFeatures->getSize(); i++) {
+  SubListOfSpeciesFeatures * item = static_cast <SubListOfSpeciesFeatures*> (mSubListOfSpeciesFeatures->get(i));
+  if (item->getId() == sid) {
+      result = item;
+  }
+      }
+
+      if (result != NULL) {
+    mSubListOfSpeciesFeatures->remove(i);
+      }
+  }
+
+  return result;
+}
+
 int
-ListOfSpeciesFeatures::addSubListOfSpeciesFeatures(ListOfSpeciesFeatures* losf)
+ListOfSpeciesFeatures::addSubListOfSpeciesFeatures(SubListOfSpeciesFeatures* losf)
 {
   if (losf == NULL)
   {
@@ -1077,7 +1305,7 @@ ListOfSpeciesFeatures::addSubListOfSpeciesFeatures(ListOfSpeciesFeatures* losf)
   else
   {
     unsigned int num = getNumSubListOfSpeciesFeatures();
-    static_cast<ListOfSpeciesFeatures*>(losf)->setIsSubList();
+//    static_cast<SubListOfSpeciesFeatures*>(losf)->setIsSubList();
     mSubListOfSpeciesFeatures->add(losf);
     connectToChild();
     if (getNumSubListOfSpeciesFeatures() == num + 1)
@@ -1099,56 +1327,6 @@ ListOfSpeciesFeatures::getNumSubListOfSpeciesFeatures() const
   return mSubListOfSpeciesFeatures->getSize();
 }
 
- /*
- * Returns the value of the "relation" attribute of this ListOfSpeciesFeatures.
- */
-Relation_t 
-ListOfSpeciesFeatures::getRelation() const
-{
-  return mRelation;
-}
-
-
-/*
- * Predicate returning @c true or @c false depending on whether this
- * ListOfSpeciesFeatures's "relation" attribute has been set.
- */
-bool 
-ListOfSpeciesFeatures::isSetRelation() const
-{
-  return (mRelation != MULTI_RELATION_UNKNOWN);
-}
-
-
-/*
- * Sets the value of the "relation" attribute of this ListOfSpeciesFeatures.
- */
-int 
-ListOfSpeciesFeatures::setRelation(Relation_t relation)
-{
-  if (SpeciesFeature_isValidRelation(relation) == 0)
-  {
-    mRelation = MULTI_RELATION_UNKNOWN;
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else
-  {
-    mRelation = relation;
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-}
-
-
-/*
- * Unsets the value of the "relation" attribute of this ListOfSpeciesFeatures.
- */
-int 
-ListOfSpeciesFeatures::unsetRelation()
-{
-  mRelation = MULTI_RELATION_UNKNOWN;
-  return LIBSBML_OPERATION_SUCCESS;
-}
-
 
 /*
  * Returns the XML element name of this object
@@ -1157,15 +1335,7 @@ const std::string&
 ListOfSpeciesFeatures::getElementName () const
 {
   static const string name = "listOfSpeciesFeatures";
-  static const string name1 = "subListOfSpeciesFeatures";
-  if (getIsSubList() == false)
-  {
-    return name;
-  }
-  else
-  {
-    return name1;
-  }
+  return name;
 }
 
 
@@ -1210,8 +1380,7 @@ ListOfSpeciesFeatures::createObject(XMLInputStream& stream)
   else if ( name == "subListOfSpeciesFeatures")
   {
     MULTI_CREATE_NS(multins, getSBMLNamespaces());
-    object = new ListOfSpeciesFeatures(multins);
-    static_cast<ListOfSpeciesFeatures*>(object)->setIsSubList();
+    object = new SubListOfSpeciesFeatures(multins);
     mSubListOfSpeciesFeatures->add(object);
     delete multins;
   }
@@ -1255,9 +1424,6 @@ void
 ListOfSpeciesFeatures::addExpectedAttributes(ExpectedAttributes& attributes)
 {
   SBase::addExpectedAttributes(attributes);
-
-  attributes.add("relation");
-
 }
 /** @endcond */
 
@@ -1267,34 +1433,6 @@ ListOfSpeciesFeatures::readAttributes (const XMLAttributes& attributes,
                                   const ExpectedAttributes& expectedAttributes)
 {
   SBase::readAttributes(attributes, expectedAttributes);
-  
-  //
-  // bindingStatus string   ( use = "optional" )
-  //
-  std::string relation;
-  bool assigned = attributes.readInto("relation", relation, 
-                                   getErrorLog(), false);
-
-  if (assigned == true)
-  {
-    // check string is not empty
-
-    if (relation.empty() == true)
-    {
-      logEmptyString(relation, getLevel(), getVersion(),
-                                    "<ListOfSpeciesFeatures>");
-    }
-    else 
-    {
-       mRelation = Relation_fromString( relation.c_str() );
-       if (SpeciesFeature_isValidRelation(mRelation) == 0)
-       {
-          getErrorLog()->logPackageError("multi", MultiLofSpeFtrs_RelationAtt_Ref,
-                       getPackageVersion(), getLevel(), getVersion(), "",
-                       getLine(), getColumn());
-       }
-    }
-  }
 }
 /** @endcond */
 
@@ -1302,8 +1440,7 @@ ListOfSpeciesFeatures::readAttributes (const XMLAttributes& attributes,
 void 
 ListOfSpeciesFeatures::writeAttributes (XMLOutputStream& stream) const
 {
-  if (isSetRelation() == true)
-    stream.writeAttribute("relation", getPrefix(), Relation_toString(mRelation));
+
 }
 /** @endcond */
 
@@ -1318,8 +1455,7 @@ ListOfSpeciesFeatures::writeElements (XMLOutputStream& stream) const
 
   for (unsigned int i = 0; i < mSubListOfSpeciesFeatures->getSize(); i++)
   {
-    static_cast<ListOfSpeciesFeatures*>(mSubListOfSpeciesFeatures->get(i))->
-      SBase::write(stream);
+    static_cast<SubListOfSpeciesFeatures*>(mSubListOfSpeciesFeatures->get(i))->write(stream);
   }
 
 }
@@ -1328,9 +1464,11 @@ ListOfSpeciesFeatures::writeElements (XMLOutputStream& stream) const
 void
 ListOfSpeciesFeatures::connectToChild()
 {
+  ListOf::connectToChild();
+
   for (unsigned int i = 0; i < mSubListOfSpeciesFeatures->getSize(); i++)
   {
-    static_cast<SBase*>(mSubListOfSpeciesFeatures->get(i))
+    static_cast<SubListOfSpeciesFeatures*>(mSubListOfSpeciesFeatures->get(i))
       ->connectToParent(this);
   }
 }
@@ -1392,6 +1530,20 @@ SpeciesFeature_getId(SpeciesFeature_t * sf)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+char *
+SpeciesFeature_getName(SpeciesFeature_t * cr)
+{
+  if (cr == NULL)
+    return NULL;
+
+  return cr->getName().empty() ? NULL : safe_strdup(cr->getName().c_str());
+}
+
+
 /**
  * 
  */
@@ -1442,6 +1594,17 @@ SpeciesFeature_isSetId(SpeciesFeature_t * sf)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesFeature_isSetName(SpeciesFeature_t * cr)
+{
+  return (cr != NULL) ? static_cast<int>(cr->isSetName()) : 0;
+}
+
+
 /**
  * 
  */
@@ -1486,6 +1649,17 @@ SpeciesFeature_setId(SpeciesFeature_t * sf, const char * id)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesFeature_setName(SpeciesFeature_t * cr, const char * name)
+{
+  return (cr != NULL) ? cr->setName(name) : LIBSBML_INVALID_OBJECT;
+}
+
+
 /**
  * 
  */
@@ -1530,6 +1704,17 @@ SpeciesFeature_unsetId(SpeciesFeature_t * sf)
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesFeature_unsetName(SpeciesFeature_t * cr)
+{
+  return (cr != NULL) ? cr->unsetName() : LIBSBML_INVALID_OBJECT;
+}
+
+
 /**
  * 
  */
@@ -1601,80 +1786,6 @@ ListOfSpeciesFeatures_removeById(ListOf_t * lo, const char * sid)
   return (sid != NULL) ? static_cast <ListOfSpeciesFeatures *>(lo)->remove(sid) : NULL;
 }
 
-static
-const char* RELATION_STRINGS[] =
-{
-    "and"
-  , "or"
-  , "not"
-  , "unknown"
-};
-
-
-LIBSBML_EXTERN
-int 
-SpeciesFeature_isValidRelation(Relation_t relation)
-{
-  int max = MULTI_RELATION_UNKNOWN;
-
-  if (relation < MULTI_RELATION_AND || relation >= max)
-  {
-    return 0;
-  }
-  else
-  {
-    return 1;
-  }
-}
-
-LIBSBML_EXTERN
-int 
-SpeciesFeature_isValidRelationString(const char* s)
-{
-  return SpeciesFeature_isValidRelation(Relation_fromString(s));
-}
-
-
-
-
-LIBSBML_EXTERN
-const char* 
-Relation_toString(Relation_t relation)
-{
-  int max = MULTI_RELATION_UNKNOWN;
-
-  if (relation < MULTI_RELATION_AND || relation >= max)
-  {
-    return NULL;
-  }
-
-  return RELATION_STRINGS[relation];
-}
-
-
-LIBSBML_EXTERN
-Relation_t 
-Relation_fromString(const char* s)
-{
-  if (s == NULL) 
-  {
-    return MULTI_RELATION_UNKNOWN;
-  }
-
-  int max = MULTI_RELATION_UNKNOWN;
-  for (int i = 0; i < max; i++)
-  {
-    if (strcmp(RELATION_STRINGS[i], s) == 0)
-    {
-      return (Relation_t)i;
-    }
-  }
-  return MULTI_RELATION_UNKNOWN;
-}
-
-
-
-
 
 LIBSBML_CPP_NAMESPACE_END
 
