@@ -9,7 +9,7 @@
  *
  * Copyright (C) 2009-2013 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
- *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *
  * Copyright (C) 2006-2008 by the California Institute of Technology,
  *     Pasadena, CA, USA 
@@ -56,8 +56,8 @@ class LIBSBML_EXTERN SpeciesFeatureType : public SBase
 
 protected:
 
-  std::string   mId;
-  std::string   mName;
+////  std::string   mId;
+////  std::string   mName;
   unsigned int  mOccur;
   bool          mIsSetOccur;
   ListOfPossibleSpeciesFeatureValues   mPossibleSpeciesFeatureValues;
@@ -215,7 +215,7 @@ public:
    *
    * @return the value of the "occur" attribute of this SpeciesFeatureType as a unsigned integer.
    */
-  virtual const unsigned int getOccur() const;
+  virtual unsigned int getOccur() const;
 
 
   /**
