@@ -117,6 +117,26 @@ SpeciesTypeComponentMapInProduct::~SpeciesTypeComponentMapInProduct ()
 }
 
 
+/*
+ * Returns the value of the "id" attribute of this SpeciesTypeComponentMapInProduct.
+ */
+const std::string&
+SpeciesTypeComponentMapInProduct::getId() const
+{
+  return mId;
+}
+
+
+/*
+ * Returns the value of the "name" attribute of this SpeciesTypeComponentMapInProduct.
+ */
+const std::string&
+SpeciesTypeComponentMapInProduct::getName() const
+{
+  return mName;
+}
+
+
 /*
  * Returns the value of the "reactant" attribute of this SpeciesTypeComponentMapInProduct.
  */
@@ -147,6 +167,26 @@ SpeciesTypeComponentMapInProduct::getProductComponent() const
 }
 
 
+/*
+ * Returns true/false if id is set.
+ */
+bool
+SpeciesTypeComponentMapInProduct::isSetId() const
+{
+  return (mId.empty() == false);
+}
+
+
+/*
+ * Returns true/false if name is set.
+ */
+bool
+SpeciesTypeComponentMapInProduct::isSetName() const
+{
+  return (mName.empty() == false);
+}
+
+
 /*
  * Returns true/false if reactant is set.
  */
@@ -177,6 +217,27 @@ SpeciesTypeComponentMapInProduct::isSetProductComponent() const
 }
 
 
+/*
+ * Sets id and returns value indicating success.
+ */
+int
+SpeciesTypeComponentMapInProduct::setId(const std::string& id)
+{
+  return SyntaxChecker::checkAndSetSId(id, mId);
+}
+
+
+/*
+ * Sets name and returns value indicating success.
+ */
+int
+SpeciesTypeComponentMapInProduct::setName(const std::string& name)
+{
+  mName = name;
+  return LIBSBML_OPERATION_SUCCESS;
+}
+
+
 /*
  * Sets reactant and returns value indicating success.
  */
@@ -231,6 +292,44 @@ SpeciesTypeComponentMapInProduct::setProductComponent(const std::string& product
 }
 
 
+/*
+ * Unsets id and returns value indicating success.
+ */
+int
+SpeciesTypeComponentMapInProduct::unsetId()
+{
+  mId.erase();
+
+  if (mId.empty() == true)
+  {
+    return LIBSBML_OPERATION_SUCCESS;
+  }
+  else
+  {
+    return LIBSBML_OPERATION_FAILED;
+  }
+}
+
+
+/*
+ * Unsets name and returns value indicating success.
+ */
+int
+SpeciesTypeComponentMapInProduct::unsetName()
+{
+  mName.erase();
+
+  if (mName.empty() == true)
+  {
+    return LIBSBML_OPERATION_SUCCESS;
+  }
+  else
+  {
+    return LIBSBML_OPERATION_FAILED;
+  }
+}
+
+
 /*
  * Unsets reactant and returns value indicating success.
  */
@@ -518,6 +617,43 @@ SpeciesTypeComponentMapInProduct::readAttributes (const XMLAttributes& attribute
 
   bool assigned = false;
 
+  //
+  // id SId  ( use = "optional" )
+  //
+  assigned = attributes.readInto("id", mId);
+
+   if (assigned == true)
+  {
+    // check string is not empty and correct syntax
+
+    if (mId.empty() == true)
+    {
+      logEmptyString(mId, getLevel(), getVersion(), "<SpeciesTypeComponentMapInProduct>");
+    }
+    else if (SyntaxChecker::isValidSBMLSId(mId) == false && getErrorLog() != NULL)
+    {
+        std::string details = "The syntax of the attribute id='" + mId + "' does not conform.";
+        getErrorLog()->logPackageError("multi", MultiInvSIdSyn,
+                   getPackageVersion(), sbmlLevel, sbmlVersion, details,
+                   getLine(), getColumn());
+    }
+  }
+
+  //
+  // name string   ( use = "optional" )
+  //
+  assigned = attributes.readInto("name", mName);
+
+  if (assigned == true)
+  {
+    // check string is not empty
+
+    if (mName.empty() == true)
+    {
+      logEmptyString(mName, getLevel(), getVersion(), "<SpeciesTypeComponentMapInProduct>");
+    }
+  }
+
   //
   // reactant SIdRef   ( use = "required" )
   //
@@ -871,6 +1007,34 @@ SpeciesTypeComponentMapInProduct_clone(SpeciesTypeComponentMapInProduct_t * stcm
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+char *
+SpeciesTypeComponentMapInProduct_getId(SpeciesTypeComponentMapInProduct_t * cr)
+{
+  if (cr == NULL)
+    return NULL;
+
+  return cr->getId().empty() ? NULL : safe_strdup(cr->getId().c_str());
+}
+
+
+/**
+ *
+ */
+LIBSBML_EXTERN
+char *
+SpeciesTypeComponentMapInProduct_getName(SpeciesTypeComponentMapInProduct_t * cr)
+{
+  if (cr == NULL)
+    return NULL;
+
+  return cr->getName().empty() ? NULL : safe_strdup(cr->getName().c_str());
+}
+
+
 /**
  * 
  */
@@ -913,6 +1077,28 @@ SpeciesTypeComponentMapInProduct_getProductComponent(SpeciesTypeComponentMapInPr
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentMapInProduct_isSetId(SpeciesTypeComponentMapInProduct_t * cr)
+{
+  return (cr != NULL) ? static_cast<int>(cr->isSetId()) : 0;
+}
+
+
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentMapInProduct_isSetName(SpeciesTypeComponentMapInProduct_t * cr)
+{
+  return (cr != NULL) ? static_cast<int>(cr->isSetName()) : 0;
+}
+
+
 /**
  * 
  */
@@ -946,6 +1132,28 @@ SpeciesTypeComponentMapInProduct_isSetProductComponent(SpeciesTypeComponentMapIn
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentMapInProduct_setId(SpeciesTypeComponentMapInProduct_t * cr, const char * id)
+{
+  return (cr != NULL) ? cr->setId(id) : LIBSBML_INVALID_OBJECT;
+}
+
+
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentMapInProduct_setName(SpeciesTypeComponentMapInProduct_t * cr, const char * name)
+{
+  return (cr != NULL) ? cr->setName(name) : LIBSBML_INVALID_OBJECT;
+}
+
+
 /**
  * 
  */
@@ -979,6 +1187,28 @@ SpeciesTypeComponentMapInProduct_setProductComponent(SpeciesTypeComponentMapInPr
 }
 
 
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentMapInProduct_unsetId(SpeciesTypeComponentMapInProduct_t * cr)
+{
+  return (cr != NULL) ? cr->unsetId() : LIBSBML_INVALID_OBJECT;
+}
+
+
+/**
+ *
+ */
+LIBSBML_EXTERN
+int
+SpeciesTypeComponentMapInProduct_unsetName(SpeciesTypeComponentMapInProduct_t * cr)
+{
+  return (cr != NULL) ? cr->unsetName() : LIBSBML_INVALID_OBJECT;
+}
+
+
 /**
  * 
  */
