@@ -42,12 +42,10 @@ LIBSBML_CPP_NAMESPACE_BEGIN
  */
 SpeciesTypeInstance::SpeciesTypeInstance (unsigned int level, unsigned int version, unsigned int pkgVersion)
   : SBase(level, version)
-   ,mId ("")
-   ,mName ("")
+////   ,mId ("")
+////   ,mName ("")
    ,mSpeciesType ("")
    ,mCompartmentReference ("")
-   ,mOccur (SBML_INT_MAX)
-   ,mIsSetOccur (false)
 {
   // set an SBMLNamespaces derived object of this package
   setSBMLNamespacesAndOwn(new MultiPkgNamespaces(level, version, pkgVersion));
@@ -59,12 +57,10 @@ SpeciesTypeInstance::SpeciesTypeInstance (unsigned int level, unsigned int versi
  */
 SpeciesTypeInstance::SpeciesTypeInstance (MultiPkgNamespaces* multins)
   : SBase(multins)
-   ,mId ("")
-   ,mName ("")
+////   ,mId ("")
+////   ,mName ("")
    ,mSpeciesType ("")
    ,mCompartmentReference ("")
-   ,mOccur (SBML_INT_MAX)
-   ,mIsSetOccur (false)
 {
   // set the element namespace of this object
   setElementNamespace(multins->getURI());
@@ -79,12 +75,10 @@ SpeciesTypeInstance::SpeciesTypeInstance (MultiPkgNamespaces* multins)
  */
 SpeciesTypeInstance::SpeciesTypeInstance (const SpeciesTypeInstance& orig)
   : SBase(orig)
-  , mId  ( orig.mId)
-  , mName  ( orig.mName)
+//  , mId  ( orig.mId)
+//  , mName  ( orig.mName)
   , mSpeciesType  ( orig.mSpeciesType)
   , mCompartmentReference  ( orig.mCompartmentReference)
-  , mOccur  ( orig.mOccur)
-  , mIsSetOccur  ( orig.mIsSetOccur)
 {
 }
 
@@ -102,8 +96,6 @@ SpeciesTypeInstance::operator=(const SpeciesTypeInstance& rhs)
     mName  = rhs.mName;
     mSpeciesType  = rhs.mSpeciesType;
     mCompartmentReference  = rhs.mCompartmentReference;
-    mOccur  = rhs.mOccur;
-    mIsSetOccur  = rhs.mIsSetOccur;
   }
   return *this;
 }
@@ -167,16 +159,6 @@ SpeciesTypeInstance::getCompartmentReference() const
 }
 
 
-/*
- * Returns the value of the "occur" attribute of this SpeciesTypeInstance.
- */
-unsigned int
-SpeciesTypeInstance::getOccur() const
-{
-  return mOccur;
-}
-
-
 /*
  * Returns true/false if id is set.
  */
@@ -217,16 +199,6 @@ SpeciesTypeInstance::isSetCompartmentReference() const
 }
 
 
-/*
- * Returns true/false if occur is set.
- */
-bool
-SpeciesTypeInstance::isSetOccur() const
-{
-  return mIsSetOccur;
-}
-
-
 /*
  * Sets id and returns value indicating success.
  */
@@ -284,18 +256,6 @@ SpeciesTypeInstance::setCompartmentReference(const std::string& compartmentRefer
 }
 
 
-/*
- * Sets occur and returns value indicating success.
- */
-int
-SpeciesTypeInstance::setOccur(unsigned int occur)
-{
-  mOccur = occur;
-  mIsSetOccur = true;
-  return LIBSBML_OPERATION_SUCCESS;
-}
-
-
 /*
  * Unsets id and returns value indicating success.
  */
@@ -372,32 +332,13 @@ SpeciesTypeInstance::unsetCompartmentReference()
 }
 
 
-/*
- * Unsets occur and returns value indicating success.
- */
-int
-SpeciesTypeInstance::unsetOccur()
-{
-  mOccur = SBML_INT_MAX;
-  mIsSetOccur = false;
-
-  if (isSetOccur() == false)
-  {
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-  else
-  {
-    return LIBSBML_OPERATION_FAILED;
-  }
-}
-
-
 /*
  * rename attributes that are SIdRefs or instances in math
  */
 void
 SpeciesTypeInstance::renameSIdRefs(const std::string& oldid, const std::string& newid)
 {
+  SBase::renameSIdRefs(oldid, newid);
   if (isSetSpeciesType() == true && mSpeciesType == oldid)
   {
     setSpeciesType(newid);
@@ -446,9 +387,6 @@ SpeciesTypeInstance::hasRequiredAttributes () const
   if (isSetSpeciesType() == false)
     allPresent = false;
 
-  if (isSetOccur() == false)
-    allPresent = false;
-
   return allPresent;
 }
 
@@ -530,7 +468,6 @@ SpeciesTypeInstance::addExpectedAttributes(ExpectedAttributes& attributes)
   attributes.add("name");
   attributes.add("speciesType");
   attributes.add("compartmentReference");
-  attributes.add("occur");
 }
 
 
@@ -648,7 +585,7 @@ SpeciesTypeInstance::readAttributes(const XMLAttributes& attributes,
   else
     {
       std::string message = "Multi attribute 'id' is missing.";
-      getErrorLog()->logPackageError("multi", MultiUnknownError,
+      getErrorLog()->logPackageError("multi", MultiSptIns_AllowedMultiAtts,
           getPackageVersion(), sbmlLevel, sbmlVersion, message,
           getLine(), getColumn());
     }
@@ -695,7 +632,7 @@ SpeciesTypeInstance::readAttributes(const XMLAttributes& attributes,
   else
     {
       std::string message = "Multi attribute 'speciesType' is missing.";
-      getErrorLog()->logPackageError("multi", MultiUnknownError,
+      getErrorLog()->logPackageError("multi", MultiSptIns_AllowedMultiAtts,
           getPackageVersion(), sbmlLevel, sbmlVersion, message,
           getLine(), getColumn());
     }
@@ -725,35 +662,6 @@ SpeciesTypeInstance::readAttributes(const XMLAttributes& attributes,
         }
     }
 
-  //
-  // occur unsigned int   ( use = "required" )
-  //
-  numErrs = getErrorLog()->getNumErrors();
-  mIsSetOccur = attributes.readInto("occur", mOccur);
-
-  if (mIsSetOccur == false)
-    {
-      if (getErrorLog() != NULL)
-        {
-          if (getErrorLog()->getNumErrors() == numErrs + 1
-              && getErrorLog()->contains(numErrs))
-            {
-              std::string message = getErrorLog()->getError(
-                  XMLAttributeTypeMismatch)->getMessage();
-              getErrorLog()->remove(XMLAttributeTypeMismatch);
-              getErrorLog()->logPackageError("multi", MultiSptIns_OccAtt_Ref,
-                  getPackageVersion(), sbmlLevel, sbmlVersion, message,
-                  getLine(), getColumn());
-            }
-          else
-            {
-              std::string message = "Multi attribute 'occur' is missing.";
-              getErrorLog()->logPackageError("multi", MultiUnknownError,
-                  getPackageVersion(), sbmlLevel, sbmlVersion, message,
-                  getLine(), getColumn());
-            }
-        }
-    }
 }
 
 
@@ -782,9 +690,6 @@ SpeciesTypeInstance::writeAttributes (XMLOutputStream& stream) const
   if (isSetCompartmentReference() == true)
     stream.writeAttribute("compartmentReference", getPrefix(), mCompartmentReference);
 
-  if (isSetOccur() == true)
-    stream.writeAttribute("occur", getPrefix(), mOccur);
-
   SBase::writeExtensionAttributes(stream);
 
 }
@@ -1083,17 +988,6 @@ SpeciesTypeInstance_getCompartmentReference(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
-LIBSBML_EXTERN
-unsigned int
-SpeciesTypeInstance_getOccur(SpeciesTypeInstance_t * sti)
-{
-  return (sti != NULL) ? sti->getOccur() : SBML_INT_MAX;
-}
-
-
 /**
  * 
  */
@@ -1138,17 +1032,6 @@ SpeciesTypeInstance_isSetCompartmentReference(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
-LIBSBML_EXTERN
-int
-SpeciesTypeInstance_isSetOccur(SpeciesTypeInstance_t * sti)
-{
-  return (sti != NULL) ? static_cast<int>(sti->isSetOccur()) : 0;
-}
-
-
 /**
  * 
  */
@@ -1193,17 +1076,6 @@ SpeciesTypeInstance_setCompartmentReference(SpeciesTypeInstance_t * sti, const c
 }
 
 
-/**
- * 
- */
-LIBSBML_EXTERN
-int
-SpeciesTypeInstance_setOccur(SpeciesTypeInstance_t * sti, unsigned int occur)
-{
-  return (sti != NULL) ? sti->setOccur(occur) : LIBSBML_INVALID_OBJECT;
-}
-
-
 /**
  * 
  */
@@ -1248,17 +1120,6 @@ SpeciesTypeInstance_unsetCompartmentReference(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
-LIBSBML_EXTERN
-int
-SpeciesTypeInstance_unsetOccur(SpeciesTypeInstance_t * sti)
-{
-  return (sti != NULL) ? sti->unsetOccur() : LIBSBML_INVALID_OBJECT;
-}
-
-
 /**
  * 
  */
