@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2016 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -99,5 +99,64 @@ addConstraint(new VConstraintDelay91020(*this));
 addConstraint(new VConstraintInitialAssignment91020(*this));
 
 addConstraint(new VConstraintConstraint91020(*this));
+
+addConstraint(new VConstraintUnit98001(*this));
+
+addConstraint(new VConstraintFunctionDefinition98002(*this));
+
+addConstraint(new VConstraintInitialAssignment98002(*this));
+
+addConstraint(new VConstraintAssignmentRule98002(*this));
+
+addConstraint(new VConstraintRateRule98002(*this));
+
+addConstraint(new VConstraintAlgebraicRule98002(*this));
+
+addConstraint(new VConstraintConstraint98002(*this));
+
+addConstraint(new VConstraintKineticLaw98002(*this));
+
+addConstraint(new VConstraintTrigger98002(*this));
+
+addConstraint(new VConstraintDelay98002(*this));
+
+addConstraint(new VConstraintPriority98002(*this));
+
+addConstraint(new VConstraintEventAssignment98002(*this));
+
+addConstraint(new ReportEmptyListOf(98003, *this));
+
+addConstraint(new VConstraintFunctionDefinition98004(*this));
+
+addConstraint(new VConstraintInitialAssignment98004(*this));
+
+addConstraint(new VConstraintAssignmentRule98004(*this));
+
+addConstraint(new VConstraintRateRule98004(*this));
+
+addConstraint(new VConstraintAlgebraicRule98004(*this));
+
+addConstraint(new VConstraintConstraint98004(*this));
+
+addConstraint(new VConstraintKineticLaw98004(*this));
+
+addConstraint(new VConstraintTrigger98004(*this));
+
+addConstraint(new VConstraintDelay98004(*this));
+
+addConstraint(new VConstraintPriority98004(*this));
+
+addConstraint(new VConstraintEventAssignment98004(*this));
+
+addConstraint(new VConstraintEvent98005(*this));
+
+addConstraint(new NumericArgsMathCheck(98006, *this));
+
+addConstraint(new IdNameNewOnSBase(98007, *this));
+
+addConstraint(new VConstraintReaction98008(*this));
+
+addConstraint(new VConstraintInitialAssignment98009(*this));
+
 /** @endcond */
 
