@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2017 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -56,7 +56,7 @@ LIBSBML_CPP_NAMESPACE_BEGIN
 
 static const char* PREAMBLE =
     "The arguments of the MathML logical operators 'and', 'or', 'xor', and "
-    "'not' must have boolean values. (References: L2V2 Section 3.5.8.)";
+    "'not' must have Boolean values. (References: L2V2 Section 3.5.8.)";
 
 
 /*
@@ -94,6 +94,8 @@ LogicalArgsMathCheck::getPreamble ()
 void
 LogicalArgsMathCheck::checkMath (const Model& m, const ASTNode& node, const SBase & sb)
 {
+  // does not apply in L3V2
+  if (m.getLevel() == 3 && m.getVersion() > 1) return;
 
   ASTNodeType_t type = node.getType();
 
@@ -128,14 +130,18 @@ LogicalArgsMathCheck::checkMath (const Model& m, const ASTNode& node, const SBas
   * If not, an error message is logged.
   */
 void 
-LogicalArgsMathCheck::checkMathFromLogical (const Model& m, const ASTNode& node, 
+LogicalArgsMathCheck::checkMathFromLogical (const Model&m, const ASTNode& node, 
                                                 const SBase & sb)
 {
   unsigned int n;
   
   for (n = 0; n < node.getNumChildren(); n++)
   {
-    if (!node.getChild(n)->isBoolean())
+    if (node.getChild(n)->isUserFunction())
+    {
+      checkMath(m, *(node.getChild(n)), sb);
+    }
+    else if (!node.getChild(n)->isBoolean())
     {
       logMathConflict(node, sb);
     }
@@ -176,7 +182,7 @@ LogicalArgsMathCheck::getMessage (const ASTNode& node, const SBase& object)
     }
     break;
   }
-  msg << "uses an argument to a logical operator that is not boolean.";
+  msg << "uses an argument to a logical operator that is not Boolean.";
   safe_free(formula);
 
   return msg.str();
